#############################################################################
##
##  PackageInfo.g for the GAP 4 package CTBlocks                Thomas Breuer
##
SetPackageInfo( rec(
PackageName :=
  "CTBlocks",
Version :=
  "0.9.5",
MyWWWHome :=
  "https://www.math.rwth-aachen.de/~Thomas.Breuer",
Subtitle :=
  "Blocks of Character Tables",
Date :=
  # "20/12/2013", # Version 0.9.0
  # "26/12/2013", # Version 0.9.1
  # "04/01/2014", # Version 0.9.2
  # "13/02/2014", # Version 0.9.3
  # "18/11/2017", # Version 0.9.4
  "31/05/2022",   # Version 0.9.5
License :=
  "GPL-3.0-or-later",
PackageWWWHome :=
  Concatenation( ~.MyWWWHome, "/", LowercaseString( ~.PackageName ) ),
ArchiveURL :=
  Concatenation( ~.PackageWWWHome, "/", LowercaseString( ~.PackageName ),
                 "-", ~.Version ),
ArchiveFormats :=
  ".tar.gz",
Persons := [
  rec(
    LastName := "Breuer",
    FirstNames := "Thomas",
    IsAuthor := true,
    IsMaintainer := true,
    Email := "sam@math.rwth-aachen.de",
    WWWHome := ~.MyWWWHome,
    Place := "Aachen",
    Institution := "Lehrstuhl für Algebra und Zahlentheorie, RWTH Aachen",
    PostalAddress := Concatenation( [
      "Thomas Breuer\n",
      "Lehrstuhl für Algebra und Zahlentheorie\n",
      "Pontdriesch 14/16\n",
      "52062 Aachen\n",
      "Germany"
      ] ),
  ),
  ],
Status :=
  "deposited",
README_URL :=
  Concatenation( ~.PackageWWWHome, "/README" ),
PackageInfoURL :=
  Concatenation( ~.PackageWWWHome, "/PackageInfo.g" ),
AbstractHTML := Concatenation( [
  "The package contains functions for studying p-blocks of groups via ",
  "their character tables, and data about several perfect groups, ",
  "in particular about central extensions of sporadic simple groups."
  ] ),
PackageDoc := rec(
  BookName :=
    "CTBlocks",
  ArchiveURLSubset :=
    [ "doc", "htm" ],
  HTMLStart :=
    "doc/chap0.html",
  PDFFile :=
    "doc/manual.pdf",
  SixFile :=
    "doc/manual.six",
  LongTitle :=
    "Blocks of Character Tables",
  ),
Dependencies := rec(
  GAP :=
    ">= 4.11.0",                  # because of CTblLib 1.3
  OtherPackagesLoadedInAdvance := [
      [ "AtlasRep", ">= 2.0" ],   # because we notify a data extension,
                                  # and we want to use the Json interface
      [ "CTblLib", ">= 1.3.1" ],  # because of tables and 'StringStandardTable'
                                  # (load in advance because of group constr.)
    ],
  NeededOtherPackages := [
      [ "gapdoc", ">= 1.6.2" ],   # want extended 'InitialSubstringUTF8String'
    ],
  SuggestedOtherPackages :=
    [ [ "mfer", ">= 1.0" ],       # because of some useful representations
      [ "TomLib", ">= 1.2.3" ],   # because of the access to groups,
                                  # (version supporting std. gens. info)
      [ "Browse", ">= 1.8" ],     # because of database attributes
    ],                            # and overview functions
  ExternalConditions :=
    [],
  ),
AvailabilityTest :=
  ReturnTrue,
TestFile :=
  "tst/testauto.g",
Keywords :=
  [ "p-blocks", "p-weights", "radical p-subgroups", "sporadic simple groups",
    "Alperin-McKay conjecture", "Alperin's weight conjecture" ],
 )
 );

#############################################################################
##
#E

