#############################################################################
##
#W  blocks.gd            GAP 4 package CTBlocks                 Thomas Breuer
##
#Y  Copyright (C)  2013,   Lehrstuhl D für Mathematik,  RWTH Aachen,  Germany
##


#############################################################################
##
#I  InfoBlocks
##
##  <#GAPDoc Label="InfoBlocks">
##  <ManSection>
##  <InfoClass Name="InfoBlocks"/>
##
##  <Description>
##  Currently only the info levels <M>0</M>, <M>1</M>, and <M>2</M>
##  are supported.
##  If the level is <M>1</M> then info messages about <K>fail</K> results
##  are printed.
##  If the level is <M>2</M> then additionally info messages about the
##  progress of computations
##  are printed.
##  The default info level of <Ref Var="InfoBlocks"/> is zero.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareInfoClass( "InfoBlocks" );

SetInfoLevel( InfoBlocks, 0 );


#############################################################################
##
##  Section "Defect, defect classes, and defect groups"
##
##  <#GAPDoc Label="intro:defect">
##  Let <M>G</M> be a group and <M>p</M> be a prime integer.
##  For any positive integer <M>m</M>, let <M>m_p</M> denote the highest
##  power of <M>p</M> that divides <M>m</M>.
##  <P/>
##  The <M>p</M>-<E>defect groups</E> of a conjugacy class <M>C</M> of
##  <M>G</M> are the Sylow <M>p</M>-subgroups of the centralizers in <M>G</M>
##  of the elements in <M>C</M>.
##  <P/>
##  Let <M>t</M> be the character table of the group <M>G</M>,
##  <M>p</M> be a prime integer,
##  and <M>B</M> be a <M>p</M>-block of <M>t</M>.
##  The <E>defect</E> of <M>B</M> is defined as the integer
##  <Display Mode="M">
##  \max\{ \log_p( |G|_p / \chi(1)_p ); \chi \in &Irr;(B) \},
##  </Display>
##  see <Cite Key="Nav98" Where="Def. 3.15"/> and
##  <Cite Key="LP10" Where="Def. 4.4.12"/>.
##  <P/>
##  A class <M>g^G</M> of <M>p</M>-regular elements in <M>G</M> is called
##  a <E>defect class</E> of <M>B</M> if both <M>\lambda_B((g^G)^+)</M> and
##  <M>(a_g)^*</M> are nonzero,
##  where <M>\lambda_B</M> is the central character of <M>B</M>
##  (see <Ref Attr="CentralCharacter" Label="for a block object"/>)
##  and <M>a_g</M> is the coefficient of the Osima idempotent of <M>B</M>
##  at <M>g</M> (see
##  <Ref Attr="CoefficientsOfOsimaIdempotent" Label="for a block object"/>).
##  Note that the set of defect classes of a block depends on the choice of
##  the <M>p</M>-modular system,
##  see Section <Ref Sect="sect:theor"/>.
##  <P/>
##  The <M>p</M>-<E>defect groups</E> of a <M>B</M> are the defect groups of
##  the defect classes of <M>B</M>.
##  Equivalently, the defect groups of <M>B</M> are the defect groups of
##  arbitrary algebraic conjugates of defect classes of <M>B</M>.
##  (<Cite Key="Isa76" Where="p. 280"/> defines the defect of a block via the
##  order of its defect group &ndash;if the defect group has order
##  <M>p^d</M> then the defect is <M>d</M>.
##  Then it is shown in <Cite Key="Isa76" Where="Theorem 15.41"/> that this
##  definition coincides with the above one.)
##  <P/>
##  By the lemma from Section <Ref Subsect="subsect:numtheor"/>,
##  the set of algebraic conjugates of the defect classes of a <M>p</M>-block
##  does <E>not</E> depend on the choice of the <M>p</M>-modular system
##  and can be computed without computing <M>p</M>-modular reductions of
##  cyclotomic numbers.
##  In order to see this, we fix a class <M>C = g^G</M> of <M>p</M>-regular
##  elements of order <M>m</M> in <M>G</M>,
##  and ask whether at least one algebraic conjugate of <M>C</M>
##  is a defect class of a given <M>p</M>-block <M>B</M>.
##  This is <E>not</E> the case if and only if
##  <M>\lambda_B((h^G)^+) \cdot (a_h)^* = 0</M> holds for all algebraic
##  conjugates <M>h</M> of <M>g</M>, that is, for all generators <M>h</M> of
##  the cyclic group <M>\langle g \rangle</M>.
##  Choose <M>\chi \in &Irr;(B)</M> and set
##  <M>\beta = \omega_{{\chi}}((g^G)^+) \cdot a_g</M>.
##  Then there is an integer <M>n</M> coprime to <M>p</M> such that
##  <M>\alpha = n \cdot \beta</M> is an algebraic integer,
##  and <M>\alpha^* = 0</M> holds if and only if
##  <M>\beta^* = \lambda_B((g^G)^+) \cdot (a_g)^* = 0</M> holds.
##  Thus no algebraic conjugate of <M>C</M> is a defect class of <M>B</M>
##  if and only if <M>\alpha / p \in &ZZ;[\zeta_m]</M> holds.
##  <#/GAPDoc>
##


#############################################################################
##
#A  Defect( <B> )
#O  Defect( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="Defect">
##  <ManSection>
##  <Heading>Defect</Heading>
##  <Attr Name="Defect" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="Defect" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  For a block that is either given by a block object <A>B</A> or as the
##  <A>b</A>-th <A>p</A>-block of the character table <A>tbl</A>,
##  <Ref Attr="Defect" Label="for a block object"/>
##  returns the defect of this block,
##  see above.
##  <P/>
##  <Example><![CDATA[
##  gap> Defect( Block( CharacterTable( "A5" ), 2, 1 ) );
##  2
##  gap> Defect( CharacterTable( "A5" ), 2, 1 );
##  2
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute( "Defect", IsBlockOfCharacterTable );

DeclareOperation( "Defect", [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#A  ClassPositionsOfDefectClasses( <B> )
#O  ClassPositionsOfDefectClasses( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="ClassPositionsOfDefectClasses">
##  <ManSection>
##  <Heading>ClassPositionsOfDefectClasses</Heading>
##  <Attr Name="ClassPositionsOfDefectClasses" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="ClassPositionsOfDefectClasses" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  Let <A>tbl</A> be an ordinary character table,
##  <A>p</A> be a prime integer,
##  and <A>b</A> be a positive integer denoting the number of
##  a <A>p</A>-block of <A>tbl</A>.
##  Alternatively, let <A>B</A> be the <A>b</A>-th <A>p</A>-block of
##  <A>tbl</A>.
##  <P/>
##  <Ref Attr="ClassPositionsOfDefectClasses" Label="for a block object"/>
##  returns the list of positions of those conjugacy classes of <A>tbl</A>
##  that are defect classes of its <A>b</A>-th <A>p</A>-block if &GAP; can
##  decide this with reasonable effort, and <K>fail</K> otherwise.
##  The result <K>fail</K> occurs if and only if either <A>tbl</A> has no
##  <A>b</A>-th <A>p</A>-block or if the known Conway polynomials
##  (see <Ref Func="ConwayPolynomial" BookName="ref"/>)
##  do not suffice.
##  <P/>
##  <Example><![CDATA[
##  gap> ClassPositionsOfDefectClasses( CharacterTable( "A5" ), 2, 1 );
##  [ 1 ]
##  gap> b:= Block( CharacterTable( "A5" ), 2, 2 );;
##  gap> ClassPositionsOfDefectClasses( b );
##  [ 3, 4, 5 ]
##  gap> b:= Block( CharacterTable( "J4" ), 23, 32 );;
##  gap> ClassPositionsOfDefectClasses( b );
##  #I  the Conway polynomial of degree 21 for p = 23 is not known
##  fail
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute( "ClassPositionsOfDefectClasses", IsBlockOfCharacterTable );

DeclareOperation( "ClassPositionsOfDefectClasses",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#A  ClassPositionsOfDefectClassesUpToGaloisConjugacy( <B> )
#O  ClassPositionsOfDefectClassesUpToGaloisConjugacy( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="ClassPositionsOfDefectClassesUpToGaloisConjugacy">
##  <ManSection>
##  <Heading>ClassPositionsOfDefectClassesUpToGaloisConjugacy</Heading>
##  <Attr Name="ClassPositionsOfDefectClassesUpToGaloisConjugacy" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="ClassPositionsOfDefectClassesUpToGaloisConjugacy"
##   Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  <Ref Oper="ClassPositionsOfDefectClassesUpToGaloisConjugacy"
##  Label="for a block object"/> is similar to
##  <Ref Oper="ClassPositionsOfDefectClasses" Label="for a block object"/>.
##  The result is either <K>fail</K> or the list of those class positions
##  such that at least one algebraic conjugate class is a defect class of the
##  <A>b</A>-th <A>p</A>-block of <A>tbl</A>.
##  <P/>
##  By the lemma from Section <Ref Subsect="subsect:numtheor"/>,
##  this information can be computed without using Conway polynomials,
##  thus <K>fail</K> is returned only if <A>tbl</A> has no <A>b</A>-th
##  <A>p</A>-block.
##  (Note that
##  <Ref Oper="ClassPositionsOfDefectClasses" Label="for a block object"/>
##  returns <K>fail</K> also if not all defect classes can be determined.)
##  <P/>
##  <Example><![CDATA[
##  gap> b:= Block( CharacterTable( "A5" ), 2, 2 );
##  Block( CharacterTable( "A5" ), 2, 2 )
##  gap> ClassPositionsOfDefectClassesUpToGaloisConjugacy( b );
##  [ 3, 4, 5 ]
##  gap> ClassPositionsOfDefectClassesUpToGaloisConjugacy(
##  >        CharacterTable( "J4" ), 23, 32 );
##  [ 2, 3, 5, 6, 7, 14, 15, 16, 19, 20, 34, 57, 58, 59, 60 ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute( "ClassPositionsOfDefectClassesUpToGaloisConjugacy",
    IsBlockOfCharacterTable );

DeclareOperation( "ClassPositionsOfDefectClassesUpToGaloisConjugacy",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#O  ClassPositionsOfDefectGroupOfClass( <tbl>, <p>, <c> )
##
##  <#GAPDoc Label="ClassPositionsOfDefectGroupOfClass">
##  <ManSection>
##  <Oper Name="ClassPositionsOfDefectGroupOfClass" Arg="tbl, p, c"/>
##
##  <Description>
##  Let <A>p</A> be a prime integer and <A>c</A> be the position of a
##  <A>p</A>-regular class in the ordinary character table <A>tbl</A>.
##  <Ref Oper="ClassPositionsOfDefectGroupOfClass"/> returns the list of
##  positions of those classes that contain elements of the defect groups of
##  the <A>c</A>-th class.
##  <P/>
##  Note that for a <A>p</A>-regular element <M>g</M> in a group <M>G</M>,
##  the classes of those <A>p</A>-elements in <M>G</M> that contain elements
##  in the centralizer of <M>g</M> in <M>G</M> are the <A>p</A>-parts
##  of those classes of <M>G</M> that contain the roots of <M>g</M>.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "J2" );;
##  gap> OrdersClassRepresentatives( tbl );
##  [ 1, 2, 2, 3, 3, 4, 5, 5, 5, 5, 6, 6, 7, 8, 10, 10, 10, 10, 12, 15, 
##    15 ]
##  gap> ClassPositionsOfDefectGroupOfClass( tbl, 2, 4 );
##  [ 1, 2, 6 ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "ClassPositionsOfDefectGroupOfClass",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#A  ClassPositionsOfDefectGroupOfBlock( <B> )
#O  ClassPositionsOfDefectGroupOfBlock( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="ClassPositionsOfDefectGroupOfBlock">
##  <ManSection>
##  <Heading>ClassPositionsOfDefectGroupOfBlock</Heading>
##  <Attr Name="ClassPositionsOfDefectGroupOfBlock" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="ClassPositionsOfDefectGroupOfBlock" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  Let <A>tbl</A> be an ordinary character table,
##  <A>p</A> be a prime integer,
##  and <A>b</A> be a positive integer denoting the number of
##  a <A>p</A>-block of <A>tbl</A>.
##  Alternatively, let <A>B</A> be the <A>b</A>-th <A>p</A>-block of
##  <A>tbl</A>.
##  <P/>
##  <Ref Oper="ClassPositionsOfDefectGroupOfBlock"
##  Label="for a block object"/> returns the list of positions of those
##  classes that contain elements of the defect groups of
##  the <A>b</A>-th <A>p</A>-block of <A>tbl</A>.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "J2" );;
##  gap> ClassPositionsOfDefectGroupOfBlock( Block( tbl, 2, 1 ) );
##  [ 1, 2, 3, 6, 14 ]
##  gap> ClassPositionsOfDefectGroupOfBlock( tbl, 2, 1 );
##  [ 1, 2, 3, 6, 14 ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute( "ClassPositionsOfDefectGroupOfBlock",
    IsBlockOfCharacterTable );

DeclareOperation( "ClassPositionsOfDefectGroupOfBlock",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#A  DefectGroup( <B> )
#O  DefectGroup( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="DefectGroup">
##  <ManSection>
##  <Heading>DefectGroup</Heading>
##  <Attr Name="DefectGroup" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="DefectGroup" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  Let <A>tbl</A> be an ordinary character table with underlying group
##  (see <Ref Func="UnderlyingGroup" Label="for character tables"
##  BookName="ref"/>) <M>G</M>, say, <A>p</A> be a prime integer,
##  and <A>b</A> be a positive integer denoting the number of
##  a <A>p</A>-block of <A>tbl</A>.
##  Alternatively, let <A>B</A> be the <A>b</A>-th <A>p</A>-block of
##  <A>tbl</A>.
##  <P/>
##  <Ref Oper="DefectGroup" Label="for a block object"/> returns
##  a defect group of the <A>b</A>-th <A>p</A>-block of <A>tbl</A>.
##  <P/>
##  If the <Ref Attr="UnderlyingGroup" Label="for character tables"
##  BookName="ref"/> is not known then <K>fail</K> is returned.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( AlternatingGroup( 5 ) );;
##  gap> DefectGroup( tbl, 2, 1 );
##  Group([ (2,3)(4,5), (2,4)(3,5) ])
##  gap> DefectGroup( Block( CharacterTable( "A5" ), 2, 1 ) );
##  fail
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute( "DefectGroup", IsBlockOfCharacterTable );

DeclareOperation( "DefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#A  NormalizerOfDefectGroup( <B> )
#O  NormalizerOfDefectGroup( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="NormalizerOfDefectGroup">
##  <ManSection>
##  <Heading>NormalizerOfDefectGroup</Heading>
##  <Attr Name="NormalizerOfDefectGroup" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="NormalizerOfDefectGroup" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  Let <A>tbl</A> be an ordinary character table with underlying group
##  (see <Ref Func="UnderlyingGroup" Label="for character tables"
##  BookName="ref"/>) <M>G</M>, say, <A>p</A> be a prime integer,
##  and <A>b</A> be a positive integer denoting the number of
##  a <A>p</A>-block of <A>tbl</A>.
##  Alternatively, let <A>B</A> be the <A>b</A>-th <A>p</A>-block of
##  <A>tbl</A>.
##  <P/>
##  <Ref Oper="NormalizerOfDefectGroup" Label="for a block object"/> returns
##  the normalizer in <M>G</M> of a defect group of the <A>b</A>-th
##  <A>p</A>-block of <A>tbl</A>.
##  <P/>
##  If the <Ref Attr="UnderlyingGroup" Label="for character tables"
##  BookName="ref"/> value is not set in <A>tbl</A> then <K>fail</K> is
##  returned.
##  <P/>
##  Note that there are methods for fetching defect normalizer information
##  from the <Package>CTBlocks</Package> database,
##  see <Ref Oper="NormalizerOfDefectGroup"
##  Label="for character table, characteristic, and block number"/>.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( AlternatingGroup( 5 ) );;
##  gap> NormalizerOfDefectGroup( tbl, 2, 1 );
##  Group([ (2,3)(4,5), (2,4)(3,5), (3,4,5) ])
##  gap> NormalizerOfDefectGroup( Block( CharacterTable( "A5" ), 2, 1 ) );
##  fail
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute( "NormalizerOfDefectGroup", IsBlockOfCharacterTable );

DeclareOperation( "NormalizerOfDefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#O  IsClassWithNormalDefectGroup( <tbl>, <p>, <c> )
##
##  <#GAPDoc Label="IsClassWithNormalDefectGroup">
##  <ManSection>
##  <Oper Name="IsClassWithNormalDefectGroup" Arg="tbl, p, c"/>
##
##  <Description>
##  returns <K>true</K> if the <A>p</A>-defect groups of the <A>c</A>-th
##  class of the ordinary character table <A>tbl</A> are normal subgroups
##  of the group of <A>tbl</A>, and <K>false</K> otherwise.
##  <P/>
##  The <A>c</A>-th class of <A>tbl</A> must be <A>p</A>-regular.
##  <P/>
##  <Example><![CDATA[
##  gap> IsClassWithNormalDefectGroup( CharacterTable( "A5" ), 2, 1 );
##  false
##  gap> IsClassWithNormalDefectGroup( CharacterTable( "A5" ), 2, 3 );
##  true
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##  
DeclareOperation( "IsClassWithNormalDefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#P  IsBlockWithNormalDefectGroup( <B> )
#O  IsBlockWithNormalDefectGroup( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="IsBlockWithNormalDefectGroup">
##  <ManSection>
##  <Heading>IsBlockWithNormalDefectGroup</Heading>
##  <Prop Name="IsBlockWithNormalDefectGroup" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="IsBlockWithNormalDefectGroup" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  returns <K>true</K> if the defect groups of the <A>b</A>-th
##  <A>p</A>-block of the ordinary character table <A>tbl</A> are normal
##  subgroups of the group of <A>tbl</A>, and <K>false</K> otherwise.
##  <P/>
##  The block can also be entered as a block object <A>B</A>.
##  <P/>
##  <Example><![CDATA[
##  gap> IsBlockWithNormalDefectGroup( CharacterTable( "A5" ), 2, 1 );
##  false
##  gap> IsBlockWithNormalDefectGroup( Block( CharacterTable( "A5" ), 2, 2 ) );
##  true
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##  
DeclareProperty( "IsBlockWithNormalDefectGroup", IsBlockOfCharacterTable );

DeclareOperation( "IsBlockWithNormalDefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#O  IsClassWithCyclicDefectGroup( <tbl>, <p>, <c> )
##
##  <#GAPDoc Label="IsClassWithCyclicDefectGroup">
##  <ManSection>
##  <Oper Name="IsClassWithCyclicDefectGroup" Arg="tbl, p, c"/>
##
##  <Description>
##  returns <K>true</K> if the <A>p</A>-defect groups of the <A>c</A>-th
##  class of the ordinary character table <A>tbl</A> are cyclic,
##  and <K>false</K> otherwise.
##  <P/>
##  The <A>c</A>-th class of <A>tbl</A> must be <A>p</A>-regular.
##  <P/>
##  <Example><![CDATA[
##  gap> IsClassWithCyclicDefectGroup( CharacterTable( "A5" ), 2, 1 );
##  false
##  gap> IsClassWithCyclicDefectGroup( CharacterTable( "A5" ), 2, 3 );
##  true
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "IsClassWithCyclicDefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#P  IsBlockWithCyclicDefectGroup( <B> )
#O  IsBlockWithCyclicDefectGroup( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="IsBlockWithCyclicDefectGroup">
##  <ManSection>
##  <Heading>IsBlockWithCyclicDefectGroup</Heading>
##  <Prop Name="IsBlockWithCyclicDefectGroup" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="IsBlockWithCyclicDefectGroup" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  returns <K>true</K> if the defect groups of the <A>b</A>-th
##  <A>p</A>-block of the ordinary character table <A>tbl</A> are cyclic,
##  and <K>false</K> otherwise.
##  <P/>
##  The block can also be entered as a block object <A>B</A>.
##  <P/>
##  <Example><![CDATA[
##  gap> IsBlockWithCyclicDefectGroup( CharacterTable( "A5" ), 2, 1 );
##  false
##  gap> IsBlockWithCyclicDefectGroup( Block( CharacterTable( "A5" ), 2, 2 ) );
##  true
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##  
DeclareProperty( "IsBlockWithCyclicDefectGroup", IsBlockOfCharacterTable );

DeclareOperation( "IsBlockWithCyclicDefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#A  TestBlockWithAbelianDefectGroup( <B> )
#O  TestBlockWithAbelianDefectGroup( <tbl>, <p>, <b> )
#A  IsBlockWithAbelianDefectGroup( <B> )
#O  IsBlockWithAbelianDefectGroup( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="TestBlockWithAbelianDefectGroup">
##  <ManSection>
##  <Heading>TestBlockWithAbelianDefectGroup</Heading>
##  <Attr Name="TestBlockWithAbelianDefectGroup" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="TestBlockWithAbelianDefectGroup" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##  <Attr Name="IsBlockWithAbelianDefectGroup" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="IsBlockWithAbelianDefectGroup" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  Brauer's height zero conjecture states that the defect groups of a
##  <M>p</M>-block of <M>G</M> are abelian if and only if all ordinary
##  irreducible characters in the block have height zero
##  (cf. <Cite Key="LP10" Where="Conjecture 4.14.6"/>
##  or <Cite Key="Nav98" Where="p. 212"/>).
##  If this conjecture would be true then it would allow us to compute from
##  the character table of a group whether the defect groups of a block are
##  abelian.
##  <P/>
##  Currently it is known that the existence of an irreducible character of
##  positive defect implies that the defect groups of its block are
##  nonabelian (see <Cite Key="KM13"/>).
##  The other direction of the conjecture is still open,
##  and it is in general not known whether the character table of a group
##  determines which defect groups of its <M>p</M>-blocks are abelian.
##  Therefore, there is no &GAP; function
##  <Ref Attr="IsBlockWithAbelianDefectGroup" Label="for a block object"/>
##  with return values only <K>true</K> and <K>false</K>.
##  Instead,
##  <Ref Attr="TestBlockWithAbelianDefectGroup" Label="for a block object"/>
##  tests some necessary conditions and allows for a <K>fail</K> result.
##  The return value is a record as described below.
##  <P/>
##  Let <A>tbl</A> be an ordinary character table,
##  <A>p</A> be a prime integer,
##  and <A>b</A> be a positive integer denoting the number of
##  a <A>p</A>-block of <A>tbl</A>.
##  Alternatively, let <A>B</A> be the <A>b</A>-th <A>p</A>-block of
##  <A>tbl</A>.
##  <P/>
##  <Ref Attr="TestBlockWithAbelianDefectGroup" Label="for a block object"/>
##  returns a record with the following components.
##  <P/>
##  <List>
##  <Mark><C>resultHZC</C></Mark>
##  <Item>
##    <K>true</K> if all ordinary irreducible characters in the block have
##    height zero, and <K>false</K> otherwise;
##    if Brauer's height zero conjecture is true then
##    this holds if and only if the defect groups of the block are abelian,
##  </Item>
##  <Mark><C>result</C></Mark>
##  <Item>
##    <K>true</K> or <K>false</K> if the defect groups of the block can be
##    shown to be abelian or nonabelian, respectively,
##    and <K>fail</K> if the criteria listed below do not suffice for a
##    decision.
##  </Item>
##  <Mark><C>comment</C></Mark>
##  <Item>
##    a string that explains the reason for the value of the <C>result</C>
##    component.
##  </Item>
##  </List>
##  <P/>
##  The following criteria are used.
##  <P/>
##  <List>
##  <Item>
##    If not all ordinary irreducible characters in the block have
##    height zero then the defect groups of the block are nonabelian,
##    by <Cite Key="KM13"/>.
##  </Item>
##  <Item>
##    If the defect is at most <M>2</M> or if the exponent of the defect
##    groups is <M>2</M> or if the defect groups are cyclic
##    then the defect groups are abelian.
##  </Item>
##  <Item>
##    If <A>tbl</A> is <A>p</A>-solvable then the height zero conjecture
##    holds for <A>tbl</A>, by <Cite Key="GW84"/>.
##  </Item>
##  <Item>
##    If the defect groups contain elements whose centralizer order is
##    not divisible by the order of the defect groups then the defect groups
##    are nonabelian.
##  </Item>
##  <Item>
##    For a block of maximal defect,
##    <Cite Key="NST13"/> yields that the defect
##    groups (the Sylow subgroups) are abelian if and only if all ordinary
##    irreducibles in the principal <A>p</A>-block have height zero
##    and the class lengths of all
##    <A>p</A>-elements are not divisible by <A>p</A>.
##  </Item>
##  <Item>
##    For a block whose defect group is normal in the group of <A>tbl</A>,
##    the height zero conjecture is true by a theorem of Reynolds,
##    see <Cite Key="Nav98" Where="Thm. 9.23"/>.
##  </Item>
##  <Item>
##    If the Sylow <A>p</A>-subgroups the group of <A>tbl</A> are abelian
##    (which can be computed by <Cite Key="NST13"/>) then the defect groups
##    of any <A>p</A>-block are abelian.
##  </Item>
##  </List>
##  <P/>
##  <Example><![CDATA[
##  gap> TestBlockWithAbelianDefectGroup( CharacterTable( "A5" ), 2, 1 );
##  rec( comment := "defect at most 2", result := true, resultHZC := true 
##   )
##  gap> TestBlockWithAbelianDefectGroup( CharacterTable( "A6" ), 2, 1 );
##  rec( comment := "not all irreducibles have height zero", 
##    result := false, resultHZC := false )
##  gap> TestBlockWithAbelianDefectGroup( CharacterTable( "J1" ), 2, 1 );
##  rec( comment := "defect group is an el. ab. 2-group", result := true, 
##    resultHZC := true )
##  gap> TestBlockWithAbelianDefectGroup(
##  >      CharacterTable( "2.L2(23).2" ), 2, 2 );
##  rec( comment := "cyclic defect group", result := true, 
##    resultHZC := true )
##  gap> TestBlockWithAbelianDefectGroup(
##  >      CharacterTable( "U3(11)" ), 2, 2 );
##  rec( comment := "no criterion yields a decision", result := fail, 
##    resultHZC := true )
##  ]]></Example>
##  <P/>
##  More examples are shown in Section
##  <Ref Sect="sect:abelian_defect_groups"/>.
##  <P/>
##  <Ref Attr="IsBlockWithAbelianDefectGroup" Label="for a block object"/>
##  returns the value of the <C>result</C> component of the result of
##  <Ref Attr="TestBlockWithAbelianDefectGroup" Label="for a block object"/>,
##  called with the same arguments.
##  <P/>
##  <Example><![CDATA[
##  gap> IsBlockWithAbelianDefectGroup( CharacterTable( "A5" ), 2, 1 );
##  true
##  gap> IsBlockWithAbelianDefectGroup( CharacterTable( "A6" ), 2, 1 );
##  false
##  gap> IsBlockWithAbelianDefectGroup( CharacterTable( "J1" ), 2, 1 );
##  true
##  gap> IsBlockWithAbelianDefectGroup( CharacterTable( "2.L2(23).2" ),
##  >        2, 2 );
##  true
##  gap> IsBlockWithAbelianDefectGroup( CharacterTable( "U3(11)" ),
##  >        2, 2 );
##  fail
##  ]]></Example>
##  <P/>
##  (Note that technically,
##  <Ref Attr="IsBlockWithAbelianDefectGroup" Label="for a block object"/>
##  cannot be implemented via a property in the sense of &GAP;'s type system,
##  see <Ref Sect="Properties" BookName="ref"/>.
##  It is implemented via an attribute.)
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute( "TestBlockWithAbelianDefectGroup",
    IsBlockOfCharacterTable );

DeclareOperation( "TestBlockWithAbelianDefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );

DeclareAttribute( "IsBlockWithAbelianDefectGroup", IsBlockOfCharacterTable );

DeclareOperation( "IsBlockWithAbelianDefectGroup",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
##  Section "Radical <M>p</M>-subgroups"
##
##  <#GAPDoc Label="intro:radical">
##  <Index>radical <M>p</M>-subgroup</Index>
##  Let <M>D</M> be a <M>p</M>-subgroup of the group <M>G</M>,
##  where <M>p</M> is a prime integer.
##  Then <M>D</M> is called a <E>radical <M>p</M>-subgroup of <M>G</M></E>
##  if <M>D</M> is equal to the <M>p</M>-core
##  (see <Ref Oper="PCore" BookName="ref"/>)
##  of the normalizer of <M>D</M> in <M>G</M>,
##  see <Cite Key="LP10" Where="Def.&nbsp;4.7.25"/> and
##  <Cite Key="Nav98" Where="p. 84"/>.
##  <P/>
##  For example, the <M>p</M>-core of <M>G</M>
##  (see <Ref Oper="PCore" BookName="ref"/>) is radical,
##  and every radical <M>p</M>-subgroup of <M>G</M> contains
##  the <M>p</M>-core of <M>G</M>.
##  Moreover, the defect groups of <M>p</M>-blocks of <M>G</M>
##  (see Section <Ref Sect="sect:defect"/>) are radical.
##  In particular the Sylow <M>p</M>-subgroups of <M>G</M> are radical.
##  <P/>
##  Radical <M>p</M>-subgroups are important for example if one deals with
##  <M>p</M>-weights, see Section <Ref Sect="sect:pweights"/>.
##  See Section <Ref Sect="sect:radical_chains"/>
##  for computing chains of radical <M>p</M>-subgroups.
##  <#/GAPDoc>
##


#############################################################################
##
#O  IsRadicalPSubgroup( <G>, <D>, <p> )
##
##  <#GAPDoc Label="IsRadicalPSubgroup">
##  <ManSection>
##  <Oper Name="IsRadicalPSubgroup" Arg="G, D, p"/>
##
##  <Description>
##  For a subgroup <A>D</A> of the group <A>G</A>
##  (this relation is <E>not</E> checked),
##  <Ref Oper="IsRadicalPSubgroup"/> returns <K>true</K> if <A>D</A>
##  is a radical <A>p</A>-subgroup of <A>G</A> (see above),
##  and <K>false</K> otherwise.
##  <P/>
##  <Example><![CDATA[
##  gap> g:= SmallGroup( 48, 29 );;  # GL(2,3)
##  gap> syl2:= SylowSubgroup( g, 2 );;
##  gap> ccl:= ConjugacyClassesSubgroups( syl2 );;
##  gap> reps:= List( ccl, Representative );;
##  gap> List( reps, Size );
##  [ 1, 2, 2, 4, 4, 4, 8, 8, 8, 16 ]
##  gap> israd:= List( reps, r -> IsRadicalPSubgroup( g, r, 2 ) );
##  [ false, false, false, false, false, false, true, false, false, true ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "IsRadicalPSubgroup", [ IsGroup, IsGroup, IsPosInt ] );


#############################################################################
##
#O  NormalizerOfRadicalPSubgroup( <G>, <D>, <p> )
##
##  <#GAPDoc Label="NormalizerOfRadicalPSubgroup">
##  <ManSection>
##  <Oper Name="NormalizerOfRadicalPSubgroup" Arg="G, D, p"/>
##
##  <Description>
##  Let <A>D</A> be a <A>p</A>-subgroup of the group <A>G</A>,
##  where <A>p</A> is a prime integer.
##  (It is <E>not</E> checked whether <A>D</A> is actually a subgroup of
##  <A>G</A>.)
##  <Ref Func="NormalizerOfRadicalPSubgroup"/> returns <K>fail</K>
##  if <A>D</A> is not a radical <A>p</A>-subgroup of <A>G</A>,
##  see <Ref Oper="IsRadicalPSubgroup"/>,
##  and the normalizer of <A>D</A> in <A>G</A> otherwise.
##  <P/>
##  The advantage of using this function instead of
##  <Ref Oper="IsRadicalPSubgroup"/> is that the normalizer need not be
##  computed twice if one is interested in it.
##  <P/>
##  <Example><![CDATA[
##  gap> # (continuing the example for IsRadicalPSubgroup)
##  gap> norm:= List( reps, r -> NormalizerOfRadicalPSubgroup( g, r, 2 ) );
##  [ fail, fail, fail, fail, fail, fail, Group([ f1, f2, f3, f4, f5 ]), 
##    fail, fail, Group([ f1, f3, f4, f5 ]) ]
##  gap> Positions( israd, false ) = Positions( norm, fail );
##  true
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "NormalizerOfRadicalPSubgroup",
    [ IsGroup, IsGroup, IsPosInt ] );


#############################################################################
##
#F  RepresentativesRadicalPSubgroupsAndNormalizers( <G>, <p>[, <arec>] )
#F  RepresentativesRadicalPSubgroupsAndNormalizers( <tom>, <p> )
##
##  <#GAPDoc Label="RepresentativesRadicalPSubgroupsAndNormalizers">
##  <ManSection>
##  <Heading>RepresentativesRadicalPSubgroupsAndNormalizers</Heading>
##  <Func Name="RepresentativesRadicalPSubgroupsAndNormalizers"
##   Arg="G, p[, arec]"
##   Label="for group, characteristic, optionally a record"/>
##  <Func Name="RepresentativesRadicalPSubgroupsAndNormalizers"
##   Arg="tom, p"
##   Label="for table of marks and characteristic"/>
##
##  <Description>
##  Let the first argument be either a group <A>G</A>
##  or the table of marks <A>tom</A> of the group <A>G</A> such that the
##  filter <Ref Filt="IsTableOfMarksWithGens" BookName="ref"/> is set in
##  <A>tom</A>,
##  and let <A>p</A> be a prime integer.
##  <Ref Func="RepresentativesRadicalPSubgroupsAndNormalizers"
##  Label="for group, characteristic, optionally a record"/> returns a list
##  of records with the components <C>subgroup</C> and <C>normalizer</C>,
##  with the values <M>U</M> and <M>N</M>, respectively,
##  such that <M>U</M> runs over representatives of conjugacy classes of
##  radical <A>p</A>-subgroups of <A>G</A>
##  and <M>N</M> is the normalizer of <M>U</M> in <A>G</A>.
##  <P/>
##  Calling <Ref Oper="PCore" BookName="ref"/> with the arguments <M>N</M>
##  and <A>p</A> yields <M>U</M>, this value is cached inside <M>N</M>.
##  <P/>
##  If the first argument is a table of marks <A>tom</A> then also the
##  components <C>subgroupnr</C> and <C>normalizernr</C> are bound in the
##  records that are returned,
##  their values are the positions of the conjugacy classes of <M>U</M> and
##  <M>N</M> in the list of classes of subgroups of <A>tom</A>.
##  Thus these numbers can be used for fetching a conjugate of <M>U</M> and
##  <M>N</M> with <Ref Oper="RepresentativeTom" BookName="ref"/>.
##  <P/>
##  The optional argument <A>arec</A>, if given, must be a record.
##  It can be used to submit data for accelerating the computations in the
##  case that a group <A>G</A> is given as the first argument.
##  The following components of <A>arec</A> are supported.
##  <P/>
##  <List>
##  <Mark><C>Maxes</C></Mark>
##  <Item>
##    a list <M>l</M> of subgroups of <A>G</A> such that any <A>p</A>-local
##    subgroup of <A>G</A> is conjugate to a subgroup of one of the groups
##    in <M>l</M>;
##    this holds if <M>l</M> contains representatives of all classes of
##    maximal subgroups of <A>G</A>, but often a smaller list suffices,
##  </Item>
##  <Mark><C>CharacterTable</C></Mark>
##  <Item>
##    the ordinary character table of <A>G</A>.
##  </Item>
##  </List>
##  <P/>
##  The idea behind the <C>Maxes</C> component is that the union <M>L</M>,
##  say, of class representatives of <A>p</A>-radical subgroups in the
##  subgroups in the list <M>l</M> covers the classes of non-normal
##  radical <A>p</A>-subgroups of <A>G</A>:
##  If <M>U</M> is <A>p</A>-radical in <A>G</A> and not normal in <M>G</M>
##  then <M>U</M> is <A>p</A>-radical in some maximal subgroup of <A>G</A>
##  that contains <M>U</M>.
##  (If <M>U</M> is not normal in <A>G</A> then the normalizer <M>N</M> of
##  <M>U</M> in <A>G</A> is contained in some maximal subgroup <M>M</M> of
##  <A>G</A>,
##  thus <M>N</M> is also the normalizer of <M>U</M> in <M>M</M>,
##  and hence <M>U</M> is <A>p</A>-radical in <M>M</M>.)
##  Thus we can find candidates for all classes of radical <A>p</A>-subgroups
##  of <A>G</A> by omitting those subgroups from <M>L</M> that are not
##  <A>p</A>-radical in <A>G</A>,
##  omitting also duplicates in the sense that the subgroups are conjugate in
##  <A>G</A>,
##  and adding the <M>p</M>-core of <A>G</A>.
##  <P/>
##  <Example><![CDATA[
##  gap> g:= DihedralGroup( 8 );;
##  gap> RepresentativesRadicalPSubgroupsAndNormalizers( g, 2 );
##  [ rec( normalizer := Group([ f1, f2, f3 ]), 
##        subgroup := Group([ f1, f2, f3 ]) ) ]
##  gap> g:= SymmetricGroup( 5 );;
##  gap> RepresentativesRadicalPSubgroupsAndNormalizers( g, 2 );
##  [ rec( normalizer := Sym( [ 1 .. 5 ] ), subgroup := Group(()) ), 
##    rec( normalizer := Group([ (3,4,5), (3,4), (1,2) ]), 
##        subgroup := Group([ (1,2) ]) ), 
##    rec( normalizer := Group([ (1,2)(3,4), (1,3)(2,4), (2,4), (2,3) ]), 
##        subgroup := Group([ (1,3)(2,4), (1,2)(3,4) ]) ), 
##    rec( normalizer := Group([ (1,4)(2,3), (3,4), (1,2) ]), 
##        subgroup := Group([ (3,4), (1,2), (1,3)(2,4) ]) ) ]
##  gap> tom:= TableOfMarks( "A5" );;
##  gap> RepresentativesRadicalPSubgroupsAndNormalizers( tom, 2 );
##  [ rec( normalizer := Alt( [ 1 .. 5 ] ), normalizernr := 9, 
##        subgroup := Group(()), subgroupnr := 1 ), 
##    rec( normalizer := Group([ (2,4)(3,5), (2,3)(4,5), (3,4,5) ]), 
##        normalizernr := 8, subgroup := Group([ (2,3)(4,5), (2,4)(3,5) ])
##          , subgroupnr := 4 ) ]
##  ]]></Example>
##  <P/>
##  The above computation shows that the alternating group on five points
##  has exactly two classes of radical <M>2</M>-subgroups,
##  the trivial subgroup and a Klein four group contained in the stabilizer
##  of a point.
##  Thus we can restrict the computations to this point stabilizer and get
##  essentially the same result.
##  <P/>
##  <Example><![CDATA[
##  gap> g:= AlternatingGroup( 5 );;
##  gap> r:= rec( Maxes:= [ Stabilizer( g, 5 ) ],
##  >             CharacterTable:= CharacterTable( "A5" ) );;
##  gap> RepresentativesRadicalPSubgroupsAndNormalizers( g, 2, r );
##  [ rec( normalizer := Alt( [ 1 .. 5 ] ), subgroup := Group(()) ), 
##    rec( normalizer := Group([ (1,4)(2,3), (1,3)(2,4), (2,3,4) ]), 
##        subgroup := Group([ (1,4)(2,3), (1,3)(2,4) ]) ) ]
##  ]]></Example>
##  <P/>
##  The following methods are available.
##  <List>
##  <Item>
##    If a table of marks <A>tom</A> is given then we run over the classes of
##    <A>p</A>-subgroups.
##  </Item>
##  <Item>
##    If a group <A>G</A> but no record <A>arec</A> with <C>Maxes</C>
##    component is given then first we run over the classes of subgroups of a
##    Sylow <A>p</A>-subgroup of <A>G</A><M> / N</M>,
##    where <M>N</M> is the <A>p</A>-core of <A>G</A>,
##    then fuse <A>G</A><M> / N</M>-conjugate classes,
##    and finally compute preimages under the natural epimorphism from
##    <A>G</A> to <A>G</A><M> / N</M>.
##  </Item>
##  <Item>
##    If a group <A>G</A> and a record <A>arec</A> with <C>Maxes</C>
##    component <M>l</M> are given then we run over the subgroups in <M>l</M>
##    and compute, for each such subgroup <M>U</M>,
##    its <A>p</A>-radical subgroups with the above method.
##    Finally, representatives under <A>G</A>-conjugacy are computed.
##  </Item>
##  </List>
##  <P/>
##  Once <Ref Func="RepresentativesRadicalPSubgroupsAndNormalizers"
##  Label="for group, characteristic, optionally a record"/> has computed the
##  result, it is stored via the attribute
##  <C>ComputedRepresentativesRadicalPSubgroupsAndNormalizers</C>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
##  Note:
##  Not all methods guarantee that all <M>p</M>-subgroups that appear in the
##  result of the function do actually lie inside one fixed
##  Sylow <M>p</M>-subgroup of <A>G</A>.
##
DeclareAttribute( "ComputedRepresentativesRadicalPSubgroupsAndNormalizers",
    IsGroup, "mutable" );

DeclareAttribute( "ComputedRepresentativesRadicalPSubgroupsAndNormalizers",
    IsTableOfMarks and IsTableOfMarksWithGens, "mutable" );

DeclareOperation( "RepresentativesRadicalPSubgroupsAndNormalizersOp",
    [ IsGroup, IsPosInt ] );

DeclareOperation( "RepresentativesRadicalPSubgroupsAndNormalizersOp",
    [ IsGroup, IsPosInt, IsRecord ] );

DeclareOperation( "RepresentativesRadicalPSubgroupsAndNormalizersOp",
    [ IsTableOfMarks and IsTableOfMarksWithGens, IsPosInt ] );

DeclareOperation( "RepresentativesRadicalPSubgroupsAndNormalizersOp",
    [ IsTableOfMarks and IsTableOfMarksWithGens, IsPosInt, IsRecord ] );

DeclareGlobalFunction( "RepresentativesRadicalPSubgroupsAndNormalizers" );


#############################################################################
##
##  Section "Chains of radical <M>p</M>-subgroups"
##
##  <#GAPDoc Label="intro:radical_chains">
##  <Index><M>p</M>-chain</Index>
##  A <E><M>p</M>-chain of length <M>n</M></E> of the group <M>G</M> is any
##  strictly increasing chain <M>P_0 &lt; P_1 &lt; \cdots &lt; P_n</M>
##  of <M>p</M>-subgroups of <M>G</M>,
##  see <Cite Key="LP10" Where="p.&nbsp;433"/>.
##  The <E>normalizer</E> of this chain is defined as
##  <M>N_0 \cap N_1 \cap \cdots \cap N_n</M>,
##  where <M>N_i</M> is the normalizer of <M>P_i</M> in <M>G</M>.
##  <#/GAPDoc>
##


#############################################################################
##
#F  RepresentativesChainsOfRadicalPSubgroupsAndNormalizers(
#F      <G>, <p>[, <arec>] )
#F  RepresentativesChainsOfRadicalPSubgroupsAndNormalizers( <tom>, <p> )
##
##  <#GAPDoc Label="RepresentativesChainsOfRadicalPSubgroupsAndNormalizers">
##  <ManSection>
##  <Heading>RepresentativesChainsOfRadicalPSubgroupsAndNormalizers</Heading>
##  <Func Name="RepresentativesChainsOfRadicalPSubgroupsAndNormalizers"
##   Arg="G, p[, arec]"
##   Label="for group, characteristic, optionally a record"/>
##  <Func Name="RepresentativesChainsOfRadicalPSubgroupsAndNormalizers"
##   Arg="tom, p"
##   Label="for table of marks and characteristic"/>
##
##  <Description>
##  Called with the same arguments as
##  <Ref Func="RepresentativesRadicalPSubgroupsAndNormalizers"
##   Label="for group, characteristic, optionally a record"/>,
##  this function returns a list <M>l</M>, say,
##  such that <M>l[k]</M> describes those ascending <M>p</M>-chains
##  of length <M>k</M> of the group in question (that is, either <A>G</A>
##  or the underlying group of <A>tom</A>), up to conjugation in this group,
##  that start at the <M>p</M>-core <M>P</M>, say, of this group and
##  that consist of radical <M>p</M>-subgroups.
##  Each entry in <M>l[k]</M> is a record with the following components.
##  <P/>
##  <List>
##  <Mark><C>subgroups</C></Mark>
##  <Item>
##    the list <M>[ P_1, P_2, \ldots, P_k ]</M> such that
##    <M>P &lt; P_1 &lt; P_2 &lt; \cdots &lt; P_k</M>
##    is the <M>p</M>-chain in question
##    (note that the <M>p</M>-core itself does not appear in the list)
##    and
##  </Item>
##  <Mark><C>normalizer</C></Mark>
##  <Item>
##     the normalizer of the chain.
##  </Item>
##  </List>
##  <P/>
##  The function calls the function
##  <Ref Func="RepresentativesRadicalPSubgroupsAndNormalizers"
##  Label="for group, characteristic, optionally a record"/>
##  for computing class representatives of radical <M>p</M>-subgroups.
##  The optional argument <A>arec</A>, if present, is used for this purpose.
##  <P/>
##  <Example><![CDATA[
##  gap> g:= DihedralGroup( 8 );;
##  gap> RepresentativesChainsOfRadicalPSubgroupsAndNormalizers( g, 2 );
##  [  ]
##  gap> g:= SymmetricGroup( 5 );;
##  gap> RepresentativesChainsOfRadicalPSubgroupsAndNormalizers( g, 2 );
##  [ [ rec( normalizer := Group([ (3,4,5), (3,4), (1,2) ]), 
##            subgroups := [ Group([ (1,2) ]) ] ), 
##        rec( normalizer := Group([ (1,2)(3,4), (1,3)(2,4), (2,4), (2,3) 
##               ]), subgroups := [ Group([ (1,3)(2,4), (1,2)(3,4) ]) ] ),
##        rec( normalizer := Group([ (1,4)(2,3), (3,4), (1,2) ]), 
##            subgroups := [ Group([ (3,4), (1,2), (1,3)(2,4) ]) ] ) ], 
##    [ 
##        rec( normalizer := Group([ (3,4), (1,2) ]), 
##            subgroups := 
##              [ Group([ (1,2) ]), Group([ (3,4), (1,2), (1,3)(2,4) ]) ] 
##           ), 
##        rec( normalizer := Group([ (1,4)(2,3), (3,4), (1,2) ]), 
##            subgroups := [ Group([ (1,3)(2,4), (1,2)(3,4) ]), 
##                Group([ (3,4), (1,2), (1,3)(2,4) ]) ] ) ] ]
##  gap> tom:= TableOfMarks( "A5" );;
##  gap> RepresentativesChainsOfRadicalPSubgroupsAndNormalizers( tom, 2 );
##  [ [ rec( normalizer := Group([ (2,4)(3,5), (2,3)(4,5), (3,4,5) ]), 
##            subgroups := [ Group([ (2,3)(4,5), (2,4)(3,5) ]) ] ) ] ]
##  ]]></Example>
##  <P/>
##  <Ref Func="RepresentativesChainsOfRadicalPSubgroupsAndNormalizers"
##  Label="for group, characteristic, optionally a record"/> stores the
##  computed result via the attribute
##  <C>ComputedRepresentativesChainsOfRadicalPSubgroupsAndNormalizers</C>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareAttribute(
    "ComputedRepresentativesChainsOfRadicalPSubgroupsAndNormalizers",
    IsGroup, "mutable" );

DeclareAttribute(
    "ComputedRepresentativesChainsOfRadicalPSubgroupsAndNormalizers",
    IsTableOfMarks and IsTableOfMarksWithGens, "mutable" );

DeclareOperation(
    "RepresentativesChainsOfRadicalPSubgroupsAndNormalizersOp",
    [ IsGroup, IsPosInt ] );

DeclareOperation(
    "RepresentativesChainsOfRadicalPSubgroupsAndNormalizersOp",
    [ IsGroup, IsPosInt, IsRecord ] );

DeclareOperation(
    "RepresentativesChainsOfRadicalPSubgroupsAndNormalizersOp",
    [ IsTableOfMarks and IsTableOfMarksWithGens, IsPosInt ] );

DeclareOperation(
    "RepresentativesChainsOfRadicalPSubgroupsAndNormalizersOp",
    [ IsTableOfMarks and IsTableOfMarksWithGens, IsPosInt, IsRecord ] );

DeclareGlobalFunction(
    "RepresentativesChainsOfRadicalPSubgroupsAndNormalizers" );


#############################################################################
##
##  Section "<M>p</M>-Weights"
##
##  <#GAPDoc Label="intro:pweights">
##  Let <M>G</M> be a group, <M>p</M> be a prime integer,
##  <M>P</M> be a <M>p</M>-subgroup of <M>G</M>,
##  and <M>N</M> be the normalizer of <M>P</M> in <M>G</M>.
##  A <E><M>p</M>-weight</E> of <M>G</M> is defined as a pair
##  <M>(P, \psi)</M>
##  where <M>\psi</M> is an irreducible defect zero character of <M>N/P</M>,
##  that is, the <M>p</M>-parts of <M>\psi(1)</M> and <M>|N/P|</M> are
##  equal (see <Cite Key="Nav98" Where="p. 90"/>).
##  <P/>
##  The <M>p</M>-weight <M>(P, \psi)</M> is said to <E>belong</E> to the
##  <M>p</M>-block <M>B</M> of <M>G</M>
##  if the <M>p</M>-block of <M>\psi</M> induces <M>B</M>,
##  see Section <Ref Sect="sect:block_induction"/>.
##  Note that each <M>p</M>-weight belongs to a block,
##  because block induction (in the sense of Brauer) from <M>N</M> to
##  <M>G</M> is always defined,
##  see for example <Cite Key="LP10" Where="Thm. 4.7.19"/>.
##  <P/>
##  A more suitable way of representing <M>p</M>-weights in our context
##  &ndash;we want to deal with character tables instead of groups&ndash;
##  is the following.
##  <P/>
##  Let <M>t</M> be the ordinary character table of <M>G</M>
##  and <M>n</M> be the ordinary character table of <M>N</M>,
##  and if <M>n</M> is different from <M>t</M> then assume that
##  the class fusion of <M>N</M> in <M>G</M> is stored on <M>n</M>.
##  Then the <M>p</M>-weight <M>(P, \psi)</M> can be represented by the
##  pair <M>(n, \psi')</M>, where <M>\psi'</M> is the irreducible
##  character of <M>N</M> that is obtained from <M>\psi</M> by
##  inflation.
##  <P/>
##  Note that <M>P</M> is <M>p</M>-radical in <M>G</M>
##  (see Section <Ref Sect="sect:radical"/>)
##  if <M>N/P</M> has a <M>p</M>-block of defect zero,
##  thus <M>P</M> is uniquely determined by <M>N</M> as its <M>p</M>-core.
##  (To see this, note that the <M>p</M>-core of a group <M>G</M>
##  is contained in the defect groups of all <M>p</M>-blocks of <M>G</M>,
##  see <Cite Key="Nav98" Where="Thm. 4.8"/> or
##  <Cite Key="LP10" Where="Thm. 4.6.10"/>.)
##  <P/>
##  Note also that the character table of <M>N</M> determines which of its
##  conjugacy classes lie in <M>P</M>,
##  see <Ref Oper="ClassPositionsOfPCore" BookName="ref"/>.
##  <P/>
##  In order to deal also with cases where the ordinary character table of
##  <M>N</M> is not known,
##  we provide the following alternative description of <M>p</M>-weights.
##  Suppose that there is a chain
##  <M>1 &lt; K \leq P &lt; N \leq M &lt; G</M> of subgroups of <M>G</M>
##  such that <M>K</M> is normal in <M>M</M>,
##  and such that the character tables <M>m</M>, <M>m'</M>, and <M>n'</M>
##  of <M>M</M>, <M>M/K</M>, and <M>N/K</M>, respectively, are known.
##  Again we have to assume that the relevant class fusions are stored on
##  the character tables:  the fusion that describes the embedding of
##  <M>N/K</M> in <M>M/K</M> and the fusion that describes the
##  projection from <M>M</M> to <M>M/K</M>.
##  In this situation, we can represent <M>(P, \psi)</M> by the pair
##  <M>((n', m', m), \psi')</M>, where <M>\psi'</M> is the irreducible
##  character of <M>N/K</M> that is obtained from <M>\psi</M> by
##  inflation.
##  <P/>
##  Note that we can compute, for a <M>p</M>-weight described by
##  <M>((n', m', m), \psi')</M>, to which <M>p</M>-block of <M>G</M> it
##  belongs:
##  Let <M>\psi</M> be the irreducible character of <M>N</M> that is
##  obtained from <M>\psi'</M> by inflation,
##  and let <M>B</M> denote the <M>p</M>-block of <M>G</M> that is induced
##  by the <M>p</M>-block of <M>\psi</M>.
##  We compute <M>B</M> by first inducing the <M>p</M>-block of <M>\psi'</M>
##  from <M>N/K</M> to <M>M/K</M>, then taking the corresponding block
##  of <M>M</M>, and then inducing this block to <M>G</M>.
##  (Note that block induction is transitive,
##  see for example <Cite Key="LP10" Where="Exercise 4.7.3"/>).
##  <#/GAPDoc>
##


#############################################################################
##
#A  DescriptionOfPWeights( <B> )
#O  DescriptionOfPWeights( <tbl>, <p>[, <b>] )
##
##  <#GAPDoc Label="DescriptionOfPWeights">
##  <ManSection>
##  <Heading>DescriptionOfPWeights</Heading>
##  <Func Name="DescriptionOfPWeights" Arg="B"
##   Label="for a block object"/>
##  <Func Name="DescriptionOfPWeights" Arg="tbl, p[, b]"
##   Label="for character table, characteristic, and optionally block number"/>
##
##  <Description>
##  First consider the case of two arguments.
##  Let <A>tbl</A> be the ordinary character table of a group <M>G</M>, say,
##  and <A>p</A> be a prime integer.
##  <Ref Func="DescriptionOfPWeights" Label="for a block object"/> returns
##  either <K>fail</K> or a record <M>r</M> that describes the
##  <A>p</A>-weights of <M>G</M>, up to <M>G</M>-conjugacy, as follows.
##  <P/>
##  Suppose that <M>G</M> has <M>k</M> conjugacy classes of radical
##  <A>p</A>-subgroups, with representatives <M>P_1, P_2, \ldots, P_k</M>.
##  <P/>
##  The <C>normalizers</C> component of <M>r</M> is a dense list
##  <M>[ n_1, n_2, \ldots, n_k ]</M>, 
##  where <M>n_i</M> is either the character table of the
##  normalizer <M>N_i</M> of <M>P_i</M> in <M>G</M>,
##  or <M>n_i</M> is a triple <M>((n_i)', (m_i)', m_i)</M> of character tables
##  as defined above in this section, which describes this normalizer.
##  <P/>
##  The <C>characters</C> component of <M>r</M> is a dense list of the same
##  length <M>k</M>,
##  its <M>i</M>-th entry is the list of those irreducible characters
##  <M>\psi</M> of <M>n_i</M> or of <M>(n_i)'</M>, respectively,
##  with the property that <M>P_i</M> is contained in the kernel of
##  <M>\psi</M> and that <M>\psi</M> has defect zero w.r.t. the factor group
##  <M>N_i / P_i</M>.
##  <P/>
##  The <C>blocks</C> component of <M>r</M> is a dense list of the same
##  length <M>k</M>,
##  its <M>i</M>-th entry is the list of numbers of the blocks in <A>tbl</A>
##  to which the given characters <M>\psi</M> of <M>n_i</M> belong.
##  <P/>
##  Now consider the case of three arguments, where <A>b</A> is a positive
##  integer,
##  and the case of one argument <A>B</A> that stands for the <A>b</A>-th
##  <A>p</A>-block of <A>tbl</A>.
##  In these cases, the format of the result is the same,
##  but it is restricted to those classes of radical <A>p</A>-subgroups and
##  to those irreducible characters for which the induced block is <A>B</A>.
##  <P/>
##  <Example><![CDATA[
##  gap> DescriptionOfPWeights( CharacterTable( AlternatingGroup( 5 ) ), 2 );
##  rec( blocks := [ [ 2 ], [ 1, 1, 1 ] ], 
##    characters := 
##      [ [ Character( CharacterTable( Alt( [ 1 .. 5 ] ) ),
##            [ 4, 0, 1, -1, -1 ] ) ], 
##        [ Character( CharacterTable( Alt( [ 1 .. 4 ] ) ),
##            [ 1, 1, 1, 1 ] ), 
##            Character( CharacterTable( Alt( [ 1 .. 4 ] ) ),
##            [ 1, E(3)^2, E(3), 1 ] ), 
##            Character( CharacterTable( Alt( [ 1 .. 4 ] ) ),
##            [ 1, E(3), E(3)^2, 1 ] ) ] ], 
##    normalizers := [ CharacterTable( Alt( [ 1 .. 5 ] ) ), 
##        CharacterTable( Alt( [ 1 .. 4 ] ) ) ] )
##  gap> DescriptionOfPWeights( CharacterTable( AlternatingGroup( 5 ) ), 2, 1 );
##  rec( 
##    characters := 
##      [ [  ], [ Character( CharacterTable( Alt( [ 1 .. 4 ] ) ),
##            [ 1, 1, 1, 1 ] ), 
##            Character( CharacterTable( Alt( [ 1 .. 4 ] ) ),
##            [ 1, E(3)^2, E(3), 1 ] ), 
##            Character( CharacterTable( Alt( [ 1 .. 4 ] ) ),
##            [ 1, E(3), E(3)^2, 1 ] ) ] ], 
##    normalizers := [ CharacterTable( Alt( [ 1 .. 5 ] ) ), 
##        CharacterTable( Alt( [ 1 .. 4 ] ) ) ] )
##  ]]></Example>
##  <P/>
##  If the attribute
##  <Ref Attr="UnderlyingGroup" BookName="ref" Label="for character tables"/>
##  is not set in <A>tbl</A> and if the <A>p</A>-weights of <A>tbl</A> are
##  not yet known then <K>fail</K> is returned.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##

#T  mention attributes and operation !!!

DeclareAttribute( "ComputedDescriptionOfPWeights",
    IsOrdinaryTable, "mutable" );

DeclareAttribute( "DescriptionOfPWeightsAttr", IsBlockOfCharacterTable );

DeclareOperation( "DescriptionOfPWeightsOp",
    [ IsOrdinaryTable, IsPosInt ] );

DeclareOperation( "DescriptionOfPWeightsOp",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );

DeclareGlobalFunction( "DescriptionOfPWeights" );


#############################################################################
##
##  Section "Block induction"
##
##  <#GAPDoc Label="intro:block_induction">
##  <E>Block induction</E> relates certain <M>p</M>-blocks of subgroups of
##  a group <M>G</M> to <M>p</M>-blocks of <M>G</M>.
##  From the various definitions of induced blocks that appear in the
##  literature, each two of the following ones coincide when both of them
##  are defined.
##  <P/>
##  In all cases, <M>G</M> is a finite group, <M>p</M> is a prime integer,
##  <M>H</M> is a subgroup of <M>G</M>,
##  and we are given a <M>p</M>-block <M>b</M> of <M>H</M>.
##  Recall that the central character <M>\omega_b^*</M> is defined as the
##  <M>p</M>-modular reduction of the class function <M>\omega_{{\chi}}</M>,
##  for any irreducible character <M>\chi</M> in <M>b</M>.
##  <P/>
##  <List>
##  <Mark>Block induction in the sense of Brauer</Mark>
##  <Item>
##    Let <M>s_H: &Centre;(FG) \rightarrow &Centre;(FH)</M> be the
##    <M>F</M>-linear map that is defined by <M>C^+ \mapsto (C \cap H)^+</M>,
##    for each conjugacy class <M>C</M> of <M>G</M>.
##    If <M>\lambda_b \circ s_H = \lambda_B</M>
##    (see <Ref Attr="CentralCharacter" Label="for a block object"/>)
##    holds for some <M>p</M>-block <M>B</M> of <M>G</M> then we call
##    <M>B</M> the block induced by <M>b</M>, and write <M>B = b^G</M>.
##    <P/>
##    The function <Ref Oper="BrauerCorrespondent"
##    Label="for a block object and the character table of the supergroup"/>
##    can be used to compute the number of the block <M>B</M>.
##    See <Cite Key="LP10" Where="Section 4.7"/> or
##    <Cite Key="Nav98" Where="p. 86 ff."/> for details.
##    Note that <M>\lambda_b \circ s_H</M> can be computed as
##    <M>(\omega_{{\chi^G}})^*</M>, if we extend the definition of
##    <Ref Attr="CentralCharacter" BookName="ref"/> to arbitrary characters,
##    see <Cite Key="LP10" Where="Lemma 4.7.5"/>.
##  </Item>
##  <Mark><M>p</M>-regular block induction</Mark>
##  <Item>
##    For any two different <M>p</M>-blocks <M>B</M>, <M>C</M> of <M>G</M>,
##    the class functions <M>\lambda_B</M>, <M>\lambda_C</M>
##    differ at some <M>p</M>-regular class,
##    see <Cite Key="LP10" Where="Proof of Thm. 4.4.8"/>.
##    Thus it makes sense to define the induced block <M>B</M> by the
##    condition that <M>\lambda_b \circ s_H</M> and <M>\lambda_B</M>
##    coincide at <M>p</M>-regular classes
##    &ndash;if such a block <M>B</M> exists then it is unique.
##    If <M>b^G</M> is defined in the sense of Brauer then of course
##    <M>B = b^G</M> holds.
##    <P/>
##    The function <Ref Oper="PRegularCorrespondent"
##    Label="for a block object and the character table of the supergroup"/>
##    can be used to compute the number of the block <M>B</M>.
##    Note that this computation is cheaper than that of <M>b^G</M>.
##    Thus it is advisable to call <Ref Oper="PRegularCorrespondent"
##    Label="for a block object and the character table of the supergroup"/>
##    instead of <Ref Oper="BrauerCorrespondent"
##    Label="for a block object and the character table of the supergroup"/>
##    if one knows in advance that <M>b^G</M> is defined.
##  </Item>
##  <Mark>Block induction in the sense of Alperin-Burry</Mark>
##  <Item>
##    This induction concept is defined not character theoretically but
##    using the interpretation of the <M>p</M>-blocks as the indecomposable
##    direct summands of the group algebra <M>RG</M>.
##    Namely, <M>RG</M> is a <M>R[G \times G]</M>-module,
##    w. r. t. the action <M>x \cdot (g_1, g_2) = g_1^{{-1}} x g_2</M>,
##    for <M>x \in RG</M> and <M>g_1, g_2 \in G</M>.
##    Restricting this action on a block <M>B</M> of <M>RG</M> to
##    <M>R[H \times H]</M> yields a direct sum of blocks of <M>RH</M>.
##    If <M>B</M> is the unique block of <M>RG</M> with the property that
##    the block <M>b</M> of <M>RH</M> occurs as a direct summand of the
##    restriction then block induction in the sense of Alperin-Burry is
##    defined, and <M>B</M> is the induced block (see <Cite Key="AB80"/>).
##    <P/>
##    The function <Ref Oper="AlperinBurryCorrespondent"
##    Label="for a block object and the character table of the supergroup"/>
##    can be used to compute the number of the block <M>B</M>,
##    provided that the underlying groups of the character tables in question
##    are available.
##  </Item>
##  <Mark>Extended block induction</Mark>
##  <Item>
##    If <M>B</M> is the unique block of <M>G</M> with the property
##    <M>(\lambda_b \circ s_H)((f_B)^*) \not= 0</M>,
##    where <M>f_B \in &Centre;(RG)</M> is the block idempotent of <M>B</M>
##    (see <Ref Attr="CoefficientsOfOsimaIdempotent"
##    Label="for a block object"/>),
##    then we say that extended block induction (or block induction in the
##    sense of Wheeler) is defined for <M>b</M>, and <M>B</M> is the induced
##    block.
##    It is shown in <Cite Key="Whe94"/> that this condition holds if block
##    induction is defined in the sense of Brauer or of Alperin-Burry,
##    thus it provides a common generalization of these concepts.
##    (In fact it generalizes <M>p</M>-regular induction.)
##    <P/>
##    The function <Ref Oper="WheelerCorrespondent"
##    Label="for a block object and the character table of the supergroup"/>
##    can be used to compute the number of the block <M>B</M>.
##    Using <Cite Key="Whe94" Where="Lemma 1.3"/>,
##    we can compute the induced block via the equality
##    <M>(\lambda_b \circ s_H)((f_B)^*) = (\chi^B(1) / \chi^G(1))^*</M>,
##    where <M>\chi</M> is any character in <M>b</M> and <M>\chi^B</M> is
##    the restriction of <M>\chi^G</M> to <M>B</M>.
##    (With this approach, we need not compute reductions of cyclotomic
##    numbers to finite fields, cf. Section <Ref Sect="sect:theor"/>.
##  </Item>
##  </List>
##  <P/>
##  The following functions implement the above concepts of block induction.
##  In all cases, four arguments occur.
##  The first two are the ordinary character tables
##  of the two groups <M>H</M> and <M>G</M>, with <M>H \leq G</M>,
##  the third is the prime integer <M>p</M>,
##  and and the fourth is either a positive integer <M>b</M> denoting
##  the number of a <M>p</M>-block of the character table of <M>H</M>
##  or an irreducible character of <M>H</M>
##  that lies in the <M>b</M>-th <M>p</M>-block of <M>H</M>.
##  <P/>
##  Alternatively, the <M>b</M>-th <M>p</M>-block of the character table of
##  <M>H</M> can be entered via a block object <M>B</M>.
##  In this case, only the character table of <M>G</M> must be supplied in
##  addition to <M>B</M>.
##  <#/GAPDoc>
##


#T document the variants BrauerCorrespondent( tblH, tblG, p, phi )
#T resp. BrauerCorrespondent( phi, tblG, p )


#############################################################################
##
#O  BrauerCorrespondent( <B>, <tblG> )
#O  BrauerCorrespondent( <tblH>, <tblG>, <p>, <b> )
##
##  <#GAPDoc Label="BrauerCorrespondent">
##  <ManSection>
##  <Heading>BrauerCorrespondent</Heading>
##  <Oper Name="BrauerCorrespondent" Arg="B, tblG"
##   Label="for a block object and the character table of the supergroup"/>
##  <Oper Name="BrauerCorrespondent" Arg="tblH, tblG, p, b"
##   Label="for two character tables, characteristic, and block number"/>
##
##  <Description>
##  Called with the arguments as described above,
##  <Ref Oper="BrauerCorrespondent"
##  Label="for a block object and the character table of the supergroup"/>
##  returns the number of the induced <A>p</A>-block of <A>tblG</A>
##  if block induction to <A>G</A> is defined for
##  the given block in the sense of Brauer (see above),
##  otherwise <K>fail</K> is returned.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "BrauerCorrespondent",
    [ IsBlockOfCharacterTable, IsOrdinaryTable ] );

DeclareOperation( "BrauerCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsPosInt ] );

DeclareOperation( "BrauerCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsList ] );


#############################################################################
##
#O  PRegularCorrespondent( <B>, <tblG> )
#O  PRegularCorrespondent( <tblH>, <tblG>, <p>, <b> )
##
##  <#GAPDoc Label="PRegularCorrespondent">
##  <ManSection>
##  <Heading>PRegularCorrespondent</Heading>
##  <Oper Name="PRegularCorrespondent" Arg="B, tblG"
##   Label="for a block object and the character table of the supergroup"/>
##  <Oper Name="PRegularCorrespondent" Arg="tblH, tblG, p, b"
##   Label="for two character tables, characteristic, and block number"/>
##
##  <Description>
##  Called with the arguments as described above,
##  <Ref Oper="PRegularCorrespondent"
##  Label="for a block object and the character table of the supergroup"/>
##  returns the number of the induced
##  <A>p</A>-block of <M>G</M> if <A>p</A>-regular block induction to
##  <A>G</A> is defined for the given block,
##  otherwise <K>fail</K> is returned.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "PRegularCorrespondent",
    [ IsBlockOfCharacterTable, IsOrdinaryTable ] );

DeclareOperation( "PRegularCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsPosInt ] );

DeclareOperation( "PRegularCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsList ] );


#############################################################################
##
#O  AlperinBurryCorrespondent( <B>, <tblG> )
#O  AlperinBurryCorrespondent( <tblH>, <tblG>, <p>, <b> )
##
##  <#GAPDoc Label="AlperinBurryCorrespondent">
##  <ManSection>
##  <Heading>AlperinBurryCorrespondent</Heading>
##  <Oper Name="AlperinBurryCorrespondent" Arg="B, tblG"
##   Label="for a block object and the character table of the supergroup"/>
##  <Oper Name="AlperinBurryCorrespondent" Arg="tblH, tblG, p, b"
##   Label="for two character tables, characteristic, and block number"/>
##
##  <Description>
##  Called with the arguments as described above,
##  <Ref Oper="AlperinBurryCorrespondent"
##  Label="for a block object and the character table of the supergroup"/>
##  returns the number of the induced
##  <A>p</A>-block of <M>G</M> if block induction to <A>G</A> is defined for
##  the given block in the sense of Alperin-Burry (see above),
##  otherwise <K>fail</K> is returned.
##  The currently available methods require that underlying groups are stored
##  for <A>tblH</A> and <A>tblG</A>, and that these groups are compatible.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
#T  describe how to compute AlperinBurryCorrespondent
#T  see~"ConstituentsInRestriction"
#T  document *admissibility*
##
DeclareOperation( "AlperinBurryCorrespondent",
    [ IsBlockOfCharacterTable, IsOrdinaryTable ] );

DeclareOperation( "AlperinBurryCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsPosInt ] );

DeclareOperation( "AlperinBurryCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsList ] );


#############################################################################
##
#O  WheelerCorrespondent( <B>, <tblG> )
#O  WheelerCorrespondent( <tblH>, <tblG>, <p>, <b> )
##
##  <#GAPDoc Label="WheelerCorrespondent">
##  <ManSection>
##  <Heading>WheelerCorrespondent</Heading>
##  <Oper Name="WheelerCorrespondent" Arg="B, tblG"
##   Label="for a block object and the character table of the supergroup"/>
##  <Oper Name="WheelerCorrespondent" Arg="tblH, tblG, p, b"
##   Label="for two character tables, characteristic, and block number"/>
##
##  <Description>
##  Called with the arguments as described above,
##  <Ref Oper="WheelerCorrespondent"
##  Label="for a block object and the character table of the supergroup"/>
##  returns the number of the induced
##  <A>p</A>-block of <M>G</M> if extended block induction to <A>G</A> is
##  defined for the given block,
##  otherwise <K>fail</K> is returned.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation("WheelerCorrespondent",
    [ IsBlockOfCharacterTable, IsOrdinaryTable ] );

DeclareOperation("WheelerCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsPosInt ] );

DeclareOperation("WheelerCorrespondent",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsList ] );


#############################################################################
##
#F  ConstituentsInRestriction( <tblH>, <tblG>, <p>, <b> )
##
##  Let <tblH> and <tblG> be the ordinary character tables of two groups $H$
##  and $G$, respectively, with $H \leq G$, <p> a prime integer,
##  and <b> a positive integer denoting the number of a <p>-block of <tblH>.
##
##  For a (splitting) field $k$ in characteristic <p>,
##  the group algebra of $G$ over $k$ is a direct sum of blocks,
##  which can be viewed as indecomposable $G \times G$ modules;
##  we consider the restriction of this action to $H \times H$.
##  `ConstituentsInRestriction' returns the list of numbers of blocks $B$ of
##  $G$ such that the <b>-th block of $H$ is (isomorphic as $H \times H$
##  module to) a direct summand of $B_{H \times H}$
#T cite Ellers-Hill (\cite{EH92}?) for the algorithm!
##
DeclareGlobalFunction( "ConstituentsInRestriction" );


#############################################################################
##
#O  IsAdmissibleBlock( <B>, <tblG> )
#O  IsAdmissibleBlock( <tblH>, <tblG>, <p>, <b> )
##
##  Let <tblH> and <tblG> be the ordinary character tables of two groups $H$
##  and $G$, respectively, with $H \leq G$, <p> a prime integer,
##  and <b> a positive integer denoting the number of a <p>-block of <tblH>.
##  `IsAdmissibleBlock' returns `true' if for the defect group $D$ of the <b>-th
##  block of $H$, the centralizer of $D$ in $G$ is contained in $H$,
##  and `false' otherwise.
#T cite Nagao-Tsushima, definition after Theorem~5.3.5.!
##
DeclareOperation( "IsAdmissibleBlock",
    [ IsBlockOfCharacterTable, IsOrdinaryTable ] );

DeclareOperation( "IsAdmissibleBlock",
    [ IsOrdinaryTable, IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#F  MultiplicityOfBlockInRestriction( <tblH>, <tblG>, <p>, <b>, <B> )
##
##  Let <tblH> and <tblG> be the ordinary character tables of two groups $H$
##  and $G$, respectively, with $H \leq G$, <p> a prime integer,
##  and <b> and <B> two positive integers denoting the numbers of <p>-blocks
##  of <tblH> and <tblG>, respectively.
##  `MultiplicityOfBlockInRestriction' returns the multiplicity of the <b>-th
##  block of <tblH> as a direct summand in the restriction of the <B>-th
##  block of <tblG> to <tblH>, viewed as an $H \times H$ module.
##
##  If the multiplicity cannot be calculated with the criteria used then
##  `MultiplicityOfBlockInRestriction' returns `fail'.
##
DeclareGlobalFunction( "MultiplicityOfBlockInRestriction" );


#############################################################################
##
#F  IsMultiplicityOne( <tblH>, <tblG>, <p>, <b> )
##
##  Let <tblH> and <tblG> be the ordinary character tables of two groups $H$
##  and $G$, respectively, with $H \leq G$, <p> a prime integer,
##  and <b> a positive integer denoting the number of a <p>-block of <tblH>.
##  `IsMultiplicityOne' returns `true' if the multiplicity of the <b>-th
##  block of <tblH> as a direct summand in the restriction of $RG$ to <tblH>, 
##  viewed as an $H \times H$ module, is proved to be equal to $1$;
##  if the multiplicity is proved to be different from $1$ then `false' is
##  returned,
##  and if the criteria used do not suffice to decide the question
##  then `fail' is returned.
##
#T refine this to mult. in restriction of a prescribed block $B$ of $G$!
##
DeclareGlobalFunction( "IsMultiplicityOne" );


#############################################################################
##
##  Section "Reality questions about blocks"
##
##  <#GAPDoc Label="intro:reality">
##  <Index>real class</Index>
##  A conjugacy class <M>C</M> of the group <M>G</M> is called <E>real</E>
##  if <M>C</M> is equal to the class of inverses of the elements in
##  <M>C</M> (see <Ref Func="RealClasses" BookName="ref"/>).
##  <P/>
##  <Index>strongly real class</Index>
##  The class <M>C</M> is called <E>strongly real</E> if its elements are
##  either the identity in <M>G</M> or are inverted by conjugation with an
##  involution (that is, an element of order two) in <M>G</M>.
##  Equivalently,
##  the elements in a strongly real class <M>C</M> are the identity or
##  products of at most two involutions in <M>G</M>;
##  note that if <M>t \in G</M> is an involution such that
##  <M>t g = g^{{-1}} t</M> holds for <M>g \in C</M> then <M>(t g)^2</M> is
##  the identity,
##  hence <M>g = t (t g)</M> is a product of at most two involutions.
##  <P/>
##  <Index>real <M>p</M>-block</Index>
##  A <M>p</M>-block <M>B</M> of <M>G</M> is called <E>real</E> if
##  <M>&Irr;(B)</M> is invariant under complex conjugation
##  (see <Ref Attr="ComplexConjugate" BookName="ref"/>).
##  In <Cite Key="Nav98" Where="p. 75"/>, real <M>p</M>-blocks are called
##  <E>selfdual</E>.
##  <P/>
##  <Index>strongly real <M>p</M>-block</Index>
##  The <M>p</M>-block <M>B</M> is called <E>strongly real</E>
##  (see <Cite Key="Mur06"/>) if it is real and has a strongly real
##  defect class (see
##  <Ref Attr="ClassPositionsOfDefectClasses" Label="for a block object"/>).
##  Since a class is strongly real if and only if all of its algebraic
##  conjugates are strongly real, the real <M>p</M>-block <M>B</M> is
##  strongly real if and only if the set of defect classes of <M>B</M> up to
##  Galois conjugacy
##  (see <Ref Attr="ClassPositionsOfDefectClassesUpToGaloisConjugacy"
##  Label="for a block object"/>)
##  contains a strongly real class.
##  <#/GAPDoc>
##


#############################################################################
##
#O  IsRealClass( <tbl>, <c> )
##
##  <#GAPDoc Label="IsRealClass">
##  <ManSection>
##  <Oper Name="IsRealClass" Arg="tbl, c"/>
##
##  <Description>
##  Let <A>c</A> be the position of a class in the ordinary character table
##  <A>tbl</A>.
##  <Ref Oper="IsRealClass"/> returns <K>true</K>
##  if the <A>c</A>-th class in <A>tbl</A> is real,
##  and <K>false</K> otherwise.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "M22" );;
##  gap> List( [ 1 .. NrConjugacyClasses( tbl ) ], 
##  >          c -> IsRealClass( tbl, c ) );
##  [ true, true, true, true, true, true, true, false, false, true, 
##    false, false ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "IsRealClass",
    [ IsOrdinaryTable, IsPosInt ] );


#############################################################################
##
#O  IsStronglyRealClass( <tbl>, <c> )
##
##  <#GAPDoc Label="IsStronglyRealClass">
##  <ManSection>
##  <Oper Name="IsStronglyRealClass" Arg="tbl, c"/>
##
##  <Description>
##  Let <A>c</A> be the position of a class in the ordinary character table
##  <A>tbl</A>.
##  <Ref Oper="IsStronglyRealClass"/> returns <K>true</K>
##  if the <A>c</A>-th class in <A>tbl</A> is strongly real,
##  and <K>false</K> otherwise.
##  <P/>
##  This property can be checked using the class multiplication coefficients
##  of <A>tbl</A>, see
##  <Ref Oper="ClassMultiplicationCoefficient" Label="for character tables"
##  BookName="ref"/>.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "M22" );;
##  gap> List( [ 1 .. NrConjugacyClasses( tbl ) ], 
##  >          c -> IsStronglyRealClass( tbl, c ) );
##  [ true, true, true, true, true, true, true, false, false, false, 
##    false, false ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareOperation( "IsStronglyRealClass",
    [ IsOrdinaryTable, IsPosInt ] );


#############################################################################
##
#P  IsRealBlock( <B> )
#O  IsRealBlock( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="IsRealBlock">
##  <ManSection>
##  <Heading>IsRealBlock</Heading>
##  <Prop Name="IsRealBlock" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="IsRealBlock" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  returns <K>true</K> if the <A>b</A>-th
##  <A>p</A>-block of the ordinary character table <A>tbl</A>
##  is real,
##  and <K>false</K> otherwise.
##  <P/>
##  The block can also be entered as a block object <A>B</A>.
##  <P/>
##  <Example><![CDATA[
##  gap> IsRealBlock( CharacterTable( "A7" ), 5, 1 );
##  true
##  gap> IsRealBlock( Block( CharacterTable( "A7" ), 5, 2 ) );
##  false
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareProperty( "IsRealBlock", IsBlockOfCharacterTable );

DeclareOperation( "IsRealBlock",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
#P  IsStronglyRealBlock( <B> )
#O  IsStronglyRealBlock( <tbl>, <p>, <b> )
##
##  <#GAPDoc Label="IsStronglyRealBlock">
##  <ManSection>
##  <Heading>IsStronglyRealBlock</Heading>
##  <Prop Name="IsStronglyRealBlock" Arg="B"
##   Label="for a block object"/>
##  <Oper Name="IsStronglyRealBlock" Arg="tbl, p, b"
##   Label="for character table, characteristic, and block number"/>
##
##  <Description>
##  returns <K>true</K> if the <A>b</A>-th
##  <A>p</A>-block of the ordinary character table <A>tbl</A>
##  is strongly real,
##  and <K>false</K> otherwise.
##  <P/>
##  The block can also be entered as a block object <A>B</A>.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "2.A5" );;
##  gap> IsStronglyRealBlock( Block( tbl, 2, 1 ) );
##  true
##  gap> IsStronglyRealBlock( tbl, 2, 2 );
##  false
##  ]]></Example>
##  <P/>
##  The principal block is always strongly real.
##  The unique nonprincipal <M>2</M>-block of <M>2.A_5 \cong SL(2,5)</M>
##  is not strongly regular;
##  note that the corresponding defect zero block of the factor group
##  <M>A_5</M> is strongly regular.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "A5" );;
##  gap> IsStronglyRealBlock( tbl, 2, 2 );
##  true
##  ]]></Example>
##  <P/>
##  (This example can be generalized:
##  The group <M>G = SL(2,q)</M>, for odd <M>q</M>, contains exactly one
##  involution, which is central in <M>G</M>.
##  Thus <M>G</M> cannot have strongly real <M>p</M>-blocks of nonmaximal
##  defect.)
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareProperty( "IsStronglyRealBlock", IsBlockOfCharacterTable );

DeclareOperation( "IsStronglyRealBlock",
    [ IsOrdinaryTable, IsPosInt, IsPosInt ] );


#############################################################################
##
##  Section "Selecting blocks according to their invariants"
##
##  <#GAPDoc Label="intro:select_blocks">
##  The idea behind the functions described in this section is that one
##  selects those <M>p</M>-blocks from a given set that satisfy certain
##  conditions,
##  where the candidates can be given either by a list of block objects
##  or by a list of character tables whose <M>p</M>-blocks are considered,
##  for all prime divisors <M>p</M> of the group order.
##  (In the case of character tables from &GAP;'s character table library,
##  one can enter also a list of admissible names of these tables,
##  for example their
##  <Ref Attr="Identifier" Label="for character tables" BookName="ref"/>
##  values.)
##  <P/>
##  For example all <M>p</M>-blocks with positive defect or with trivial
##  kernel can be considered,
##  or more generally all <M>p</M>-blocks with prescribed values for some
##  properties and attributes that have been introduced in the previous
##  sections.
##  <P/>
##  <Ref Func="AllPBlocks"/> and <Ref Func="OnePBlock"/> return all blocks
##  and one block with the desired properties, respectively.
##  <P/>
##  <Ref Func="BlockInvariants"/> returns a record that contains a matrix of
##  block invariants (one row for each block with the desired properties)
##  and a description of the invariants corresponding to the columns.
##  <Ref Func="DisplayBlockInvariants"/> and
##  <Ref Func="BrowseBlockInvariants"/> (this function requires the
##  <Package>Browse</Package> package <Cite Key="Browse"/>)
##  show a tabular overview of these invariants.
##  The columns of these matrices of block invariants can be customized,
##  see <Ref Var="CTBlocks.BlockInvariantsColumns"/>.
##  <#/GAPDoc>
##


#############################################################################
##
#F  AllPBlocks( <source>[, <fun1>, <val1>, ...] )
##
##  <#GAPDoc Label="AllPBlocks">
##  <ManSection>
##  <Func Name="AllPBlocks" Arg="source[, fun1, val1, ...]"/>
##
##  <Description>
##  The first argument <A>source</A> must be either
##  a list <M>l_1</M> of block objects or
##  (an admissible name of) an ordinary character table <M>t</M> or
##  a list of (admissible names of) ordinary character tables <M>l_2</M>.
##  If there is just this argument then <Ref Func="AllPBlocks"/>
##  returns <M>l_1</M> or the list of all <M>p</M>-blocks of <M>t</M>,
##  for all primes dividing the order of the underlying group of <M>t</M>,
##  or the concatenation of these lists for the tables in <M>l_2</M>.
##  <P/>
##  Additional arguments <A>fun1</A>, <A>val1</A>, <A>fun2</A>, <A>val2</A>
##  <M>\ldots</M> can be used to restrict the output.
##  Each of <A>fun1</A>, <A>fun2</A>, <M>\ldots</M> must be a unary function
##  that takes a block object as its argument,
##  and each of <A>val1</A>, <A>val2</A>, <M>\ldots</M> describes the
##  admissible values, as follows:
##  The <M>i</M>-th value is admissible for the result of the <M>i</M>-th
##  function applied to a given block object if the result is either equal
##  to the value or (if the value is a list) is contained in the value or
##  (if the value is a function) yields <K>true</K> when the value is
##  applied to the result.
##  <Ref Func="AllPBlocks"/> returns the list of those blocks for which the
##  values of all functions are admissible.
##  <P/>
##  <Example><![CDATA[
##  gap> t:= CharacterTable( "A5" );;
##  gap> AllPBlocks( t );
##  [ Block( CharacterTable( "A5" ), 2, 1 ), 
##    Block( CharacterTable( "A5" ), 2, 2 ), 
##    Block( CharacterTable( "A5" ), 3, 1 ), 
##    Block( CharacterTable( "A5" ), 3, 2 ), 
##    Block( CharacterTable( "A5" ), 3, 3 ), 
##    Block( CharacterTable( "A5" ), 5, 1 ), 
##    Block( CharacterTable( "A5" ), 5, 2 ) ]
##  gap> AllPBlocks( t, Defect, 2 );
##  [ Block( CharacterTable( "A5" ), 2, 1 ) ]
##  gap> AllPBlocks( t, Defect, IsPosInt );
##  [ Block( CharacterTable( "A5" ), 2, 1 ), 
##    Block( CharacterTable( "A5" ), 3, 1 ), 
##    Block( CharacterTable( "A5" ), 5, 1 ) ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "AllPBlocks" );


#############################################################################
##
#F  OnePBlock( <source>[, <fun1>, <val1>, ...] )
##
##  <#GAPDoc Label="OnePBlock">
##  <ManSection>
##  <Func Name="OnePBlock" Arg="source[, fun1, val1, ...]"/>
##
##  <Description>
##  The difference between <Ref Func="OnePBlock"/> and
##  <Ref Func="AllPBlocks"/> is that <Ref Func="OnePBlock"/>,
##  when called with the same arguments, returns the first entry of the
##  result of <Ref Func="AllPBlocks"/> if this result is nonempty,
##  and <K>fail</K> otherwise.
##  <P/>
##  <Example><![CDATA[
##  gap> t:= CharacterTable( "A5" );;
##  gap> OnePBlock( t );
##  Block( CharacterTable( "A5" ), 2, 1 )
##  gap> OnePBlock( t, UnderlyingCharacteristic, 2 );
##  Block( CharacterTable( "A5" ), 2, 1 )
##  gap> OnePBlock( t, Defect, 2 );
##  Block( CharacterTable( "A5" ), 2, 1 )
##  gap> OnePBlock( t, Defect, IsPosInt );
##  Block( CharacterTable( "A5" ), 2, 1 )
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "OnePBlock" );


#############################################################################
##
#F  BlockInvariants( <source>[, <fun1>, <val1>, ...][, <arec>] )
##
##  <#GAPDoc Label="BlockInvariants">
##  <ManSection>
##  <Func Name="BlockInvariants" Arg="source[, fun1, val1, ...][, arec]"/>
##
##  <Description>
##  This function returns a record with the following components.
##  <P/>
##  <List>
##  <Mark><C>list</C></Mark>
##  <Item>
##    a matrix of block invariants whose rows correspond to the
##    block objects that are returned by <Ref Func="AllPBlocks"/> when this
##    function is called with the same arguments,
##  </Item>
##  <Mark><C>columns</C></Mark>
##  <Item>
##    a list af records that correspond to the columns of the matrix
##    stored in the <C>list</C> component;
##    for the format of these records,
##    see <Ref Var="CTBlocks.BlockInvariantsColumns"/>,
##  </Item>
##  <Mark><C>header</C></Mark>
##  <Item>
##    a list of strings that describe the contents.
##  </Item>
##  </List>
##  <P/>
##  For the meaning of the arguments of this function except <A>arec</A>,
##  see <Ref Func="AllPBlocks"/>.
##  The optional argument <A>arec</A> can be used to customize the columns
##  of the <C>list</C> component,
##  see <Ref Var="CTBlocks.BlockInvariantsColumns"/>.
##  <P/>
##  <Example><![CDATA[
##  gap> t:= CharacterTable( "A5" );;
##  gap> inv:= BlockInvariants( t );;
##  gap> List( inv.columns, r -> r.label );
##  [ "p", "b", "d", "k", "l", "c", "a", "n", "f", "r", "sr" ]
##  gap> Display( inv.list );
##  [ [ 2, 1, 2, 4, 3, false, true, false, true, true, true ], 
##    [ 2, 2, 0, 1, 1, true, true, true, true, true, true ], 
##    [ 3, 1, 1, 3, 2, true, true, false, true, true, true ], 
##    [ 3, 2, 0, 1, 1, true, true, true, true, true, true ], 
##    [ 3, 3, 0, 1, 1, true, true, true, true, true, true ], 
##    [ 5, 1, 1, 4, 2, true, true, false, true, true, true ], 
##    [ 5, 2, 0, 1, 1, true, true, true, true, true, true ] ]
##  ]]></Example>
##  <P/>
##  The following values of <A>fun1</A>, <A>fun2</A>, <M>\ldots</M> have the
##  additional effect that the corresponding column is omitted from the
##  <C>list</C> component if exactly one admissible value is prescribed.
##  <P/>
##  <List>
##  <Item>
##    <Ref Attr="Defect" Label="for a block object"/>
##  </Item>
##  <Item>
##    <Ref Attr="IsBlockWithAbelianDefectGroup" Label="for a block object"/>
##  </Item>
##  <Item>
##    <Ref Prop="IsBlockWithCyclicDefectGroup" Label="for a block object"/>
##  </Item>
##  <Item>
##    <Ref Prop="IsBlockWithNormalDefectGroup" Label="for a block object"/>
##  </Item>
##  <Item>
##    <Ref Prop="IsFaithful" Label="for a block object"/>
##  </Item>
##  <Item>
##    <Ref Attr="NumberOfBlock" Label="for a block object"/>
##  </Item>
##  <Item>
##    <Ref Attr="UnderlyingCharacteristic" Label="for a block object"/>
##  </Item>
##  </List>
##  <P/>
##  <Example><![CDATA[
##  gap> inv:= BlockInvariants( t, UnderlyingCharacteristic, 2 );;
##  gap> List( inv.columns, r -> r.label );
##  [ "b", "d", "k", "l", "c", "a", "n", "f", "r", "sr" ]
##  gap> Display( inv.list );
##  [ [ 1, 2, 4, 3, false, true, false, true, true, true ], 
##    [ 2, 0, 1, 1, true, true, true, true, true, true ] ]
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "BlockInvariants" );


#############################################################################
##
#F  DisplayBlockInvariants( <source>[, <fun1>, <val1>, ...][, <arec>] )
##
##  <#GAPDoc Label="DisplayBlockInvariants">
##  <ManSection>
##  <Func Name="DisplayBlockInvariants"
##   Arg="source[, fun1, val1, ...][, arec]"/>
##
##  <Description>
##  This function prints the return value of <Ref Func="BlockInvariants"/>,
##  when this function is called with the same arguments,
##  in tabular form.
##  <P/>
##  If there is just one argument then <Ref Func="DisplayBlockInvariants"/>
##  prints tabular information about the <M>p</M>-blocks of the given
##  character table(s), for all primes <M>p</M> dividing the group order(s).
##  Each row corresponds to a <M>p</M>-block,
##  and each column corresponds to a block invariant.
##  <P/>
##  (First we set the user preference <C>DisplayFunction</C> to the value
##  <C>"Print"</C> in order to make sure that only ASCII characters are shown
##  in the following example output.
##  The value will be reset at the end of the examples in this manual
##  chapter.)
##  <P/>
##  <Example><![CDATA[
##  gap> origpref:= UserPreference( "AtlasRep", "DisplayFunction" );;
##  gap> SetUserPreference( "AtlasRep", "DisplayFunction", "Print" );
##  gap> t:= CharacterTable( "2.A5" );;
##  gap> DisplayBlockInvariants( t );
##  Block invariants for 2.A5
##  
##  ----------------------------------------------
##  | p | b | d | k | l | c | a | n | f | r | sr |
##  ----------------------------------------------
##  | 2 | 1 | 3 | 7 | 3 | - | - | - | + | + |  + |
##  |   | 2 | 1 | 2 | 1 | + | + | + | + | + |  - |
##  | 3 | 1 | 1 | 3 | 2 | + | + | - | - | + |  + |
##  |   | 2 | 0 | 1 | 1 | + | + | + | - | + |  - |
##  |   | 3 | 0 | 1 | 1 | + | + | + | - | + |  - |
##  |   | 4 | 1 | 3 | 2 | + | + | - | + | + |  + |
##  |   | 5 | 0 | 1 | 1 | + | + | + | + | + |  - |
##  | 5 | 1 | 1 | 4 | 2 | + | + | - | - | + |  + |
##  |   | 2 | 0 | 1 | 1 | + | + | + | - | + |  - |
##  |   | 3 | 1 | 4 | 2 | + | + | - | + | + |  + |
##  ----------------------------------------------
##  ]]></Example>
##  <P/>
##  See <Ref Var="CTBlocks.BlockInvariantsColumns"/> for the meaning of the
##  columns.
##  <P/>
##  The meaning of the arguments <A>fun1</A>, <A>val1</A>, <A>fun2</A>,
##  <A>val2</A> <M>\ldots</M> for restricting the rows
##  and the meaning of <A>arec</A> for customizing the columns and the header
##  is the same as described for <Ref Func="BlockInvariants"/>.
##  <P/>
##  <Example><![CDATA[
##  gap> DisplayBlockInvariants( t, UnderlyingCharacteristic, 2 );;
##  Block invariants for 2.A5
##  p = 2
##  
##  ------------------------------------------
##  | b | d | k | l | c | a | n | f | r | sr |
##  ------------------------------------------
##  | 1 | 3 | 7 | 3 | - | - | - | + | + |  + |
##  | 2 | 1 | 2 | 1 | + | + | + | + | + |  - |
##  ------------------------------------------
##  ]]></Example>
##  <P/>
##  The user preference <C>DisplayFunction</C> from the package
##  <Package>AtlasRep</Package> controls whether the output is just printed
##  or shown using a pager.
##  Depending on the value of this user preference and the terminal
##  capabilities, nicer table borders may be used than are shown in the above
##  examples.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "DisplayBlockInvariants" );


#############################################################################
##
#F  BrowseBlockInvariants( <source>[, <fun1>, <val1>, ...][, <arec>] )
##
##  <#GAPDoc Label="BrowseBlockInvariants">
##  <ManSection>
##  <Func Name="BrowseBlockInvariants"
##   Arg="source[, fun1, val1, ...][, arec]"/>
##
##  <Description>
##  The arguments of this function are equal to the ones for
##  <Ref Func="DisplayBlockInvariants"/>.
##  The difference to that function is that
##  <Ref Func="BrowseBlockInvariants"/> does not print the overview to the
##  screen but shows it in a browse table, see
##  <Ref Chap="Browsing Tables in GAP using ncurses –The User Interface"
##  BookName="Browse"/>.
##  <P/>
##  <Ref Func="BrowseBlockInvariants"/> can be used only if the
##  <Package>Browse</Package> package <Cite Key="Browse"/> is loaded.
##  <P/>
##  The full functionality of the function
##  <Ref Func="NCurses.BrowseGeneric" BookName="Browse"/> is available.
##  <P/>
##  <Example><![CDATA[
##  gap> n:= [ 14, 14, 14, 14, 14, 14 ];;  # ``do nothing''
##  gap> enter:= [ NCurses.keys.ENTER ];;
##  gap> BrowseData.SetReplay( Concatenation(
##  >        "sc",                     # select the 'p' column,
##  >        "sc",                     # categorize by this column,
##  >        "X",                      # expand all categories
##  >        n,                        # wait a little
##  >        "scrrrrrrr",              # select the 'f' column
##  >        "f+", enter,              # restrict to the faithful blocks
##  >        n,                        # wait a little
##  >        "!",                      # reset the filtering etc.
##  >        n,                        # wait a little
##  >        "Q" ) );                  # and quit
##  gap> t:= CharacterTable( "2.A5" );;
##  gap> BrowseBlockInvariants( t );
##  gap> BrowseData.SetReplay( false );
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "BrowseBlockInvariants" );


#############################################################################
##
##  functions for computations with the centre of a group algebra,
##  and of the centres of p-blocks
##
##  <#GAPDoc Label="intro:centre_block_algebra">
##  The functions described in this section create algebra objects
##  (see <Ref Chap="Algebras" BookName="ref"/>) which are isomorphic with
##  the centres of blocks or group algebras.
##  The structure constants of the centre of a group algebra can be computed
##  from the character table of the group in question, see
##  <Ref Oper="ClassMultiplicationCoefficient" Label="for character tables"
##  BookName="ref"/>,
##  and the centre of a <M>p</M>-block is a suitable subalgebra.
##  <#/GAPDoc>
##


##############################################################################
##
#F  SCAlgebraCentreOfGroupAlgebra( <G>, <p> )
#F  SCAlgebraCentreOfGroupAlgebra( <G>, <F> )
#F  SCAlgebraCentreOfGroupAlgebra( <tbl>, <p> )
#F  SCAlgebraCentreOfGroupAlgebra( <tbl>, <F> )
##
##  <#GAPDoc Label="SCAlgebraCentreOfGroupAlgebra">
##  <ManSection>
##  <Heading>SCAlgebraCentreOfGroupAlgebra</Heading>
##  <Func Name="SCAlgebraCentreOfGroupAlgebra" Arg="G, p"
##   Label="for group and given characteristic"/>
##  <Func Name="SCAlgebraCentreOfGroupAlgebra" Arg="G, F"
##   Label="for group and given field of coefficients"/>
##  <Func Name="SCAlgebraCentreOfGroupAlgebra" Arg="tbl, p"
##   Label="for char. table and given characteristic"/>
##  <Func Name="SCAlgebraCentreOfGroupAlgebra" Arg="tbl, F"
##   Label="for char. table and given field of coefficients"/>
##
##  <Description>
##  Let either <A>G</A> be a group or <A>tbl</A> be
##  the ordinary character table of a group.
##  <P/>
##  If the second argument is a prime integer <A>p</A> then
##  <Ref Func="SCAlgebraCentreOfGroupAlgebra" Label="for group and given characteristic"/> returns a s. c. algebra (see
##  <Ref Sect="Constructing Algebras by Structure Constants" BookName="ref"/>
##  that is isomorphic with the centre of the group algebra of the group,
##  over the field with <A>p</A> elements.
##  <P/>
##  If the second argument is a finite field <A>F</A> then the same holds,
##  except that the coefficients domain of the algebra is <A>F</A>.
##  <P/>
##  The canonical basis of the algebra corresponds to the class sums
##  in the group; the basis vectors appear in the same ordering as in the
##  <Ref Attr="ConjugacyClasses" BookName="ref"/> value for <A>G</A> or
##  <A>tbl</A>, respectively.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "M11" );
##  CharacterTable( "M11" )
##  gap> a:= SCAlgebraCentreOfGroupAlgebra( tbl, 3 );
##  <algebra-with-one of dimension 10 over GF(3)>
##  gap> a.2 * a.3;
##  (Z(3))*C2+(Z(3))*C4+C7+C8+(Z(3))*C9+(Z(3))*C10
##  gap> Dimension( RadicalOfAlgebra( a ) );
##  8
##  gap> SCAlgebraCentreOfGroupAlgebra( tbl, GF(9) );
##  <algebra-with-one of dimension 10 over GF(3^2)>
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "SCAlgebraCentreOfGroupAlgebra" );


##############################################################################
##
#F  BlockDecompositionSCAlgebraCentreOfGroupAlgebra( <tbl>, <p> )
#F  BlockDecompositionSCAlgebraCentreOfGroupAlgebra( <tbl>, <F> )
##
##  <#GAPDoc Label="BlockDecompositionSCAlgebraCentreOfGroupAlgebra">
##  <ManSection>
##  <Heading>BlockDecompositionSCAlgebraCentreOfGroupAlgebra</Heading>
##  <Func Name="BlockDecompositionSCAlgebraCentreOfGroupAlgebra"
##   Arg="tbl, p" Label="for given characteristic"/>
##  <Func Name="BlockDecompositionSCAlgebraCentreOfGroupAlgebra"
##   Arg="tbl, F" Label="for given field of coefficients"/>
##
##  <Description>
##  Let <A>tbl</A> be an ordinary character table.
##  <P/>
##  If the second argument is a prime integer <A>p</A> then this function
##  returns a list of subalgebras of the s. c. algebra returned by
##  <Ref Func="SCAlgebraCentreOfGroupAlgebra"
##  Label="for char. table and given characteristic"/>
##  when this is called with <A>tbl</A> and the smallest field that contains
##  the <A>p</A>-modular reductions of all block idempotents
##  of <A>tbl</A> in characteristic <A>p</A>.
##  The <M>i</M>-th subalgebra corresponds to the <M>i</M>-th <A>p</A>-block
##  of <A>tbl</A>.
##  <P/>
##  If the second argument is a finite field <A>F</A> then the same holds,
##  except that the coefficients domain of the algebras is <A>F</A>.
##  If <A>F</A> is too small then <K>fail</K> is returned.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "M11" );
##  CharacterTable( "M11" )
##  gap> dec:= BlockDecompositionSCAlgebraCentreOfGroupAlgebra(
##  >              tbl, 5 );
##  [ <algebra of dimension 5 over GF(5^2)>, 
##    <algebra of dimension 1 over GF(5^2)>, 
##    <algebra of dimension 1 over GF(5^2)>, 
##    <algebra of dimension 1 over GF(5^2)>, 
##    <algebra of dimension 1 over GF(5^2)>, 
##    <algebra of dimension 1 over GF(5^2)> ]
##  gap> List( dec, Dimension );
##  [ 5, 1, 1, 1, 1, 1 ]
##  gap> dec:= BlockDecompositionSCAlgebraCentreOfGroupAlgebra(
##  >              tbl, GF(5) );
##  fail
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "BlockDecompositionSCAlgebraCentreOfGroupAlgebra" );


##############################################################################
##
#F  SCAlgebraCentreOfBlock( <tbl>, <p>, <b> )
#F  SCAlgebraCentreOfBlock( <tbl>, <F>, <b> )
##
##  <#GAPDoc Label="SCAlgebraCentreOfBlock">
##  <ManSection>
##  <Heading>SCAlgebraCentreOfBlock</Heading>
##  <Func Name="SCAlgebraCentreOfBlock"
##   Arg="tbl, p, b" Label="for given characteristic"/>
##  <Func Name="SCAlgebraCentreOfBlock"
##   Arg="tbl, F, b" Label="for given field of coefficients"/>
##
##  <Description>
##  Let <A>tbl</A> be an ordinary character table,
##  and <A>b</A> be a positive integer.
##  <P/>
##  If the second argument is a prime integer <A>p</A> then
##  <Ref Func="SCAlgebraCentreOfBlock" Label="for given characteristic"/>
##  returns a subalgebra of the s. c. algebra returned by
##  <Ref Func="SCAlgebraCentreOfGroupAlgebra" Label="for group and given characteristic"/> when this is called with
##  <A>tbl</A> and the smallest field that contains
##  the <A>p</A>-modular reductions of the <A>b</A>-th block idempotent
##  of <A>tbl</A> in characteristic <A>p</A>.
##  <P/>
##  If the second argument is a finite field <A>F</A> then the same holds,
##  except that the coefficients domain of the algebra is <A>F</A>.
##  If <A>F</A> is too small then <K>fail</K> is returned.
##  <P/>
##  <Example><![CDATA[
##  gap> tbl:= CharacterTable( "M11" );
##  CharacterTable( "M11" )
##  gap> SCAlgebraCentreOfBlock( tbl, 5, 1 );
##  <algebra of dimension 5 over GF(5)>
##  gap> SCAlgebraCentreOfBlock( tbl, 5, 3 );
##  <algebra of dimension 1 over GF(5^2)>
##  gap> SCAlgebraCentreOfBlock( tbl, GF(5), 3 );
##  fail
##  ]]></Example>
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
##
DeclareGlobalFunction( "SCAlgebraCentreOfBlock" );


#############################################################################
##
#E

